"""
This type stub file was generated by pyright.
"""

__all__ = ["Day", "BaseOffset", "BusinessDay", "BusinessMonthBegin", "BusinessMonthEnd", "BDay", "CustomBusinessDay", "CustomBusinessMonthBegin", "CustomBusinessMonthEnd", "CDay", "CBMonthEnd", "CBMonthBegin", "MonthBegin", "BMonthBegin", "MonthEnd", "BMonthEnd", "SemiMonthEnd", "SemiMonthBegin", "BusinessHour", "CustomBusinessHour", "YearBegin", "BYearBegin", "YearEnd", "BYearEnd", "QuarterBegin", "BQuarterBegin", "QuarterEnd", "BQuarterEnd", "LastWeekOfMonth", "FY5253Quarter", "FY5253", "Week", "WeekOfMonth", "Easter", "Tick", "Hour", "Minute", "Second", "Milli", "Micro", "Nano", "DateOffset"]



"""
This type stub file was generated by pyright.
"""

from contextlib import contextmanager
from typing import Any, IO, Iterator

@contextmanager
def decompress_file(path, compression) -> Iterator[IO[bytes]]:
    """
    Open a compressed file and return a file object.

    Parameters
    ----------
    path : str
        The path where the file is read from.

    compression : {'gzip', 'bz2', 'zip', 'xz', 'zstd', None}
        Name of the decompression to use

    Returns
    -------
    file object
    """
    ...

@contextmanager
def set_timezone(tz: str) -> Iterator[None]:
    """
    Context manager for temporarily setting a timezone.

    Parameters
    ----------
    tz : str
        A string representing a valid timezone.

    Examples
    --------
    >>> from datetime import datetime
    >>> from dateutil.tz import tzlocal
    >>> tzlocal().tzname(datetime(2021, 1, 1))  # doctest: +SKIP
    'IST'

    >>> with set_timezone('US/Eastern'):
    ...     tzlocal().tzname(datetime(2021, 1, 1))
    ...
    'EST'
    """
    ...

@contextmanager
def ensure_clean(filename=..., return_filelike: bool = ..., **kwargs: Any): # -> Generator[TextIOWrapper | str, None, None]:
    """
    Gets a temporary path and agrees to remove on close.

    This implementation does not use tempfile.mkstemp to avoid having a file handle.
    If the code using the returned path wants to delete the file itself, windows
    requires that no program has a file handle to it.

    Parameters
    ----------
    filename : str (optional)
        suffix of the created file.
    return_filelike : bool (default False)
        if True, returns a file-like which is *always* cleaned. Necessary for
        savefig and other functions which want to append extensions.
    **kwargs
        Additional keywords are passed to open().

    """
    ...

@contextmanager
def ensure_clean_dir() -> Iterator[str]:
    """
    Get a temporary directory path and agrees to remove on close.

    Yields
    ------
    Temporary directory path
    """
    ...

@contextmanager
def ensure_safe_environment_variables() -> Iterator[None]:
    """
    Get a context manager to safely set environment variables

    All changes will be undone on close, hence environment variables set
    within this contextmanager will neither persist nor change global state.
    """
    ...

@contextmanager
def with_csv_dialect(name, **kwargs) -> Iterator[None]:
    """
    Context manager to temporarily register a CSV dialect for parsing CSV.

    Parameters
    ----------
    name : str
        The name of the dialect.
    kwargs : mapping
        The parameters for the dialect.

    Raises
    ------
    ValueError : the name of the dialect conflicts with a builtin one.

    See Also
    --------
    csv : Python's CSV library.
    """
    ...

@contextmanager
def use_numexpr(use, min_elements=...) -> Iterator[None]:
    ...

class RNGContext:
    """
    Context manager to set the numpy random number generator speed. Returns
    to the original value upon exiting the context manager.

    Parameters
    ----------
    seed : int
        Seed for numpy.random.seed

    Examples
    --------
    with RNGContext(42):
        np.random.randn()
    """
    def __init__(self, seed) -> None:
        ...
    
    def __enter__(self) -> None:
        ...
    
    def __exit__(self, exc_type, exc_value, traceback) -> None:
        ...
    





"""
This type stub file was generated by pyright.
"""

import os
import platform
import sys
import lzma
from __future__ import annotations
from typing import TYPE_CHECKING
from pandas._typing import F
from pandas.compat.numpy import is_numpy_dev, np_version_under1p21
from pandas.compat.pyarrow import pa_version_under1p01, pa_version_under2p0, pa_version_under3p0, pa_version_under4p0, pa_version_under5p0, pa_version_under6p0, pa_version_under7p0, pa_version_under8p0, pa_version_under9p0

"""
compat
======

Cross-compatible functions for different versions of Python.

Other items:
* platform checker
"""
if TYPE_CHECKING:
    ...
PY39 = ...
PY310 = ...
PY311 = ...
PYPY = ...
IS64 = sys.maxsize > 2 ** 32
def set_function_name(f: F, name: str, cls) -> F:
    """
    Bind the name/qualname attributes of the function.
    """
    ...

def is_platform_little_endian() -> bool:
    """
    Checking if the running platform is little endian.

    Returns
    -------
    bool
        True if the running platform is little endian.
    """
    ...

def is_platform_windows() -> bool:
    """
    Checking if the running platform is windows.

    Returns
    -------
    bool
        True if the running platform is windows.
    """
    ...

def is_platform_linux() -> bool:
    """
    Checking if the running platform is linux.

    Returns
    -------
    bool
        True if the running platform is linux.
    """
    ...

def is_platform_mac() -> bool:
    """
    Checking if the running platform is mac.

    Returns
    -------
    bool
        True if the running platform is mac.
    """
    ...

def is_platform_arm() -> bool:
    """
    Checking if the running platform use ARM architecture.

    Returns
    -------
    bool
        True if the running platform uses ARM architecture.
    """
    ...

def is_ci_environment() -> bool:
    """
    Checking if running in a continuous integration environment by checking
    the PANDAS_CI environment variable.

    Returns
    -------
    bool
        True if the running in a continuous integration environment.
    """
    ...

def get_lzma_file() -> type[lzma.LZMAFile]:
    """
    Importing the `LZMAFile` class from the `lzma` module.

    Returns
    -------
    class
        The `LZMAFile` class from the `lzma` module.

    Raises
    ------
    RuntimeError
        If the `lzma` module was not imported correctly, or didn't exist.
    """
    ...

__all__ = ["is_numpy_dev", "np_version_under1p21", "pa_version_under1p01", "pa_version_under2p0", "pa_version_under3p0", "pa_version_under4p0", "pa_version_under5p0", "pa_version_under6p0", "pa_version_under7p0", "pa_version_under8p0", "pa_version_under9p0"]



"""
This type stub file was generated by pyright.
"""

"""
Reversed Operations not available in the stdlib operator module.
Defining these instead of using lambdas allows us to reference them by name.
"""
def radd(left, right):
    ...

def rsub(left, right):
    ...

def rmul(left, right):
    ...

def rdiv(left, right):
    ...

def rtruediv(left, right):
    ...

def rfloordiv(left, right):
    ...

def rmod(left, right):
    ...

def rdivmod(left, right):
    ...

def rpow(left, right):
    ...

def rand_(left, right): # -> Any:
    ...

def ror_(left, right): # -> Any:
    ...

def rxor(left, right): # -> Any:
    ...




"""
This type stub file was generated by pyright.
"""

from pandas.core.arrays import ArrowExtensionArray, ArrowStringArray, BooleanArray, Categorical, DatetimeArray, FloatingArray, IntegerArray, IntervalArray, PandasArray, PeriodArray, SparseArray, StringArray, TimedeltaArray

"""
All of pandas' ExtensionArrays.

See :ref:`extending.extension-types` for more.
"""
__all__ = ["ArrowExtensionArray", "ArrowStringArray", "BooleanArray", "Categorical", "DatetimeArray", "FloatingArray", "IntegerArray", "IntervalArray", "PandasArray", "PeriodArray", "SparseArray", "StringArray", "TimedeltaArray"]



# BoARIO : The Adaptative Regional Input Output model in python.
# Copyright (C) 2022  Samuel Juhel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import pathlib
import pymrio as pym
import pandas as pd
import pickle as pkl
import logging
import argparse
import json
import country_converter as coco
from pymrio.core.mriosystem import IOSystem

parser = argparse.ArgumentParser(description='Aggregate an EXIOBASE3 MRIO table in less sectors (an optionally less regions)')
parser.add_argument('exio_path', type=str, help='The str path to the exio3 zip file')
parser.add_argument('aggreg_path', type=str, help='The str path to the ods aggregation matrix file')
parser.add_argument('sector_names_json_path', type=str, help='The str path to the json file with the new names of the sectors')
parser.add_argument('regions_aggregator_json_path', type=str, help='The str path to the json file with the regions aggregation', nargs='?', default=None)
parser.add_argument('-o', "--output", type=str, help='The str path to save the pickled mrio to', nargs='?', default='./mrio_dump')

args = parser.parse_args()
logFormatter = logging.Formatter("%(asctime)s [%(levelname)-5.5s] %(name)s %(message)s", datefmt="%H:%M:%S")
scriptLogger = logging.getLogger("aggreg_exio3")
consoleHandler = logging.StreamHandler()
consoleHandler.setFormatter(logFormatter)

scriptLogger.addHandler(consoleHandler)
scriptLogger.setLevel(logging.INFO)
scriptLogger.propagate = False

def aggreg(exio_path, sector_aggregator_path, new_sectors_name_path, regions_aggregator_json_path=None, save_path=None):
    with pathlib.Path(new_sectors_name_path).open('r') as f:
        a = json.load(f)
    if regions_aggregator_json_path is not None:
        with pathlib.Path(regions_aggregator_json_path).open('r') as f:
            b = json.load(f)
    sec_agg_matrix = pd.read_excel(sector_aggregator_path, sheet_name="input", engine="odf", header=None).to_numpy()
    scriptLogger.info("Parsing exiobase3 from {}".format(pathlib.Path(exio_path).resolve()))
    exio3 = pym.parse_exiobase3(path=exio_path)
    # gain some diskspace and RAM by removing unused attributes
    attr = ['Z', 'Y', 'x', 'A', 'L', 'unit', 'population', 'meta', '__non_agg_attributes__', '__coefficients__', '__basic__']
    tmp = list(exio3.__dict__.keys())
    for at in tmp:
        if at not in attr:
            delattr(exio3,at)
    assert isinstance(exio3, IOSystem)
    scriptLogger.info("Done")
    scriptLogger.info("Computing the IO components")
    exio3.calc_all()
    scriptLogger.info("Done")
    scriptLogger.info("Reading aggregation matrix from sheet 'input' in file {}".format(pathlib.Path(sector_aggregator_path).absolute()))
    scriptLogger.info("Aggregating from {} to {} sectors".format(len(exio3.get_sectors()), sec_agg_matrix.shape[0]))
    exio3.aggregate(sector_agg=sec_agg_matrix)
    scriptLogger.info("Done")
    scriptLogger.info("Renaming sectors from {}".format(pathlib.Path(new_sectors_name_path).absolute()))
    exio3.rename_sectors(a)
    scriptLogger.info("Done")
    if regions_aggregator_json_path is not None:
        regions_aggregator = coco.agg_conc(original_countries=exio3.get_regions(),
                                           aggregates=b['aggregates'],
                                           missing_countries=b['missing'])
        exio3.aggregate(region_agg=regions_aggregator)
    exio3.calc_all()
    name = save_path
    scriptLogger.info("Saving to {}".format(pathlib.Path(name).absolute()))

    with open(name, 'wb') as f:
        pkl.dump(exio3, f)

if __name__ == '__main__':
    args = parser.parse_args()
    name = pathlib.Path(args.exio_path).stem
    aggreg(args.exio_path, args.aggreg_path, args.sector_names_json_path, regions_aggregator_json_path=args.regions_aggregator_json_path, save_path=args.output)



"""
This type stub file was generated by pyright.
"""

from typing import NamedTuple, TYPE_CHECKING
from pandas._typing import ArrayLike
from pandas._libs.internals import BlockPlacement
from pandas.core.internals.blocks import Block
from pandas.core.internals.managers import BlockManager

if TYPE_CHECKING:
    ...
class BlockPairInfo(NamedTuple):
    lvals: ArrayLike
    rvals: ArrayLike
    locs: BlockPlacement
    left_ea: bool
    right_ea: bool
    rblk: Block
    ...


def operate_blockwise(left: BlockManager, right: BlockManager, array_op) -> BlockManager:
    ...

def blockwise_all(left: BlockManager, right: BlockManager, op) -> bool:
    """
    Blockwise `all` reduction.
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np

"""
transforms.py is for shape-preserving functions.
"""
def shift(values: np.ndarray, periods: int, axis: int, fill_value) -> np.ndarray:
    ...




"""
This type stub file was generated by pyright.
"""

from typing import Any
from pandas.core.base import PandasObject

"""
frozen (immutable) data structures to support MultiIndexing

These are used for:

- .names (FrozenList)

"""
class FrozenList(PandasObject, list):
    """
    Container that doesn't allow setting item *but*
    because it's technically hashable, will be used
    for lookups, appropriately, etc.
    """
    def union(self, other) -> FrozenList:
        """
        Returns a FrozenList with other concatenated to the end of self.

        Parameters
        ----------
        other : array-like
            The array-like whose elements we are concatenating.

        Returns
        -------
        FrozenList
            The collection difference between self and other.
        """
        ...
    
    def difference(self, other) -> FrozenList:
        """
        Returns a FrozenList with elements from other removed from self.

        Parameters
        ----------
        other : array-like
            The array-like whose elements we are removing self.

        Returns
        -------
        FrozenList
            The collection difference between self and other.
        """
        ...
    
    __add__ = ...
    def __getitem__(self, n): # -> Self@FrozenList:
        ...
    
    def __radd__(self, other): # -> Self@FrozenList:
        ...
    
    def __eq__(self, other: Any) -> bool:
        ...
    
    __req__ = ...
    def __mul__(self, other): # -> Self@FrozenList:
        ...
    
    __imul__ = ...
    def __reduce__(self): # -> tuple[Type[Self@FrozenList], tuple[list[Unknown]]]:
        ...
    
    def __hash__(self) -> int:
        ...
    
    def __str__(self) -> str:
        ...
    
    def __repr__(self) -> str:
        ...
    
    __setitem__ = ...
    __delitem__ = ...
    pop = ...
    remove = ...





"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas._libs.tslibs import BaseOffset, Timedelta, Timestamp
from pandas._typing import npt

"""
Helper functions to generate range-like data for DatetimeArray
(and possibly TimedeltaArray/PeriodArray)
"""
def generate_regular_range(start: Timestamp | Timedelta | None, end: Timestamp | Timedelta | None, periods: int | None, freq: BaseOffset) -> npt.NDArray[np.intp]:
    """
    Generate a range of dates or timestamps with the spans between dates
    described by the given `freq` DateOffset.

    Parameters
    ----------
    start : Timedelta, Timestamp or None
        First point of produced date range.
    end : Timedelta, Timestamp or None
        Last point of produced date range.
    periods : int or None
        Number of periods in produced date range.
    freq : Tick
        Describes space between dates in produced date range.

    Returns
    -------
    ndarray[np.int64] Representing nanoseconds.
    """
    ...




"""
This type stub file was generated by pyright.
"""

import pandas._libs.parsers as parsers
from typing import Hashable, Mapping, Sequence, TYPE_CHECKING
from pandas._typing import ArrayLike, DtypeArg, DtypeObj, ReadCsvBuffer
from pandas.io.parsers.base_parser import ParserBase
from pandas import Index, MultiIndex

if TYPE_CHECKING:
    ...
class CParserWrapper(ParserBase):
    low_memory: bool
    _reader: parsers.TextReader
    def __init__(self, src: ReadCsvBuffer[str], **kwds) -> None:
        ...
    
    def close(self) -> None:
        ...
    
    def read(self, nrows: int | None = ...) -> tuple[Index | MultiIndex | None, Sequence[Hashable] | MultiIndex, Mapping[Hashable, ArrayLike],]:
        ...
    


def ensure_dtype_objs(dtype: DtypeArg | dict[Hashable, DtypeArg] | None) -> DtypeObj | dict[Hashable, DtypeObj] | None:
    """
    Ensure we have either None, a dtype object, or a dictionary mapping to
    dtype objects.
    """
    ...




"""
This type stub file was generated by pyright.
"""

from pandas._libs.lib import no_default
from pandas.core.dtypes.base import ExtensionDtype, register_extension_dtype
from pandas.core.accessor import register_dataframe_accessor, register_index_accessor, register_series_accessor
from pandas.core.algorithms import take
from pandas.core.arrays import ExtensionArray, ExtensionScalarOpsMixin

"""
Public API for extending pandas objects.
"""
__all__ = ["no_default", "ExtensionDtype", "register_extension_dtype", "register_dataframe_accessor", "register_index_accessor", "register_series_accessor", "take", "ExtensionArray", "ExtensionScalarOpsMixin"]



"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas._typing import npt

def unstack(values: np.ndarray, mask: np.ndarray, stride: int, length: int, width: int, new_values: np.ndarray, new_mask: np.ndarray) -> None:
    ...

def explode(values: npt.NDArray[np.object_]) -> tuple[npt.NDArray[np.object_], npt.NDArray[np.int64]]:
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Any
from pandas.core.ops.common import unpack_zerodim_and_defer

"""
Methods that can be shared by many array-like classes or subclasses:
    Series
    Index
    ExtensionArray
"""
REDUCTION_ALIASES = ...
class OpsMixin:
    @unpack_zerodim_and_defer("__eq__")
    def __eq__(self, other) -> bool:
        ...
    
    @unpack_zerodim_and_defer("__ne__")
    def __ne__(self, other) -> bool:
        ...
    
    @unpack_zerodim_and_defer("__lt__")
    def __lt__(self, other) -> bool:
        ...
    
    @unpack_zerodim_and_defer("__le__")
    def __le__(self, other) -> bool:
        ...
    
    @unpack_zerodim_and_defer("__gt__")
    def __gt__(self, other) -> bool:
        ...
    
    @unpack_zerodim_and_defer("__ge__")
    def __ge__(self, other) -> bool:
        ...
    
    @unpack_zerodim_and_defer("__and__")
    def __and__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__rand__")
    def __rand__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__or__")
    def __or__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__ror__")
    def __ror__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__xor__")
    def __xor__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__rxor__")
    def __rxor__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__add__")
    def __add__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__radd__")
    def __radd__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__sub__")
    def __sub__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__rsub__")
    def __rsub__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__mul__")
    def __mul__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__rmul__")
    def __rmul__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__truediv__")
    def __truediv__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__rtruediv__")
    def __rtruediv__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__floordiv__")
    def __floordiv__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__rfloordiv")
    def __rfloordiv__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__mod__")
    def __mod__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__rmod__")
    def __rmod__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__divmod__")
    def __divmod__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__rdivmod__")
    def __rdivmod__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__pow__")
    def __pow__(self, other): # -> _NotImplementedType:
        ...
    
    @unpack_zerodim_and_defer("__rpow__")
    def __rpow__(self, other): # -> _NotImplementedType:
        ...
    


def array_ufunc(self, ufunc: np.ufunc, method: str, *inputs: Any, **kwargs: Any): # -> Any | _NotImplementedType | tuple[Unknown, ...]:
    """
    Compatibility with numpy ufuncs.

    See also
    --------
    numpy.org/doc/stable/reference/arrays.classes.html#numpy.class.__array_ufunc__
    """
    ...

def dispatch_ufunc_with_out(self, ufunc: np.ufunc, method: str, *inputs, **kwargs): # -> _NotImplementedType | tuple[Unknown, ...]:
    """
    If we have an `out` keyword, then call the ufunc without `out` and then
    set the result into the given `out`.
    """
    ...

def default_array_ufunc(self, ufunc: np.ufunc, method: str, *inputs, **kwargs): # -> Any:
    """
    Fallback to the behavior we would get if we did not define __array_ufunc__.

    Notes
    -----
    We are assuming that `self` is among `inputs`.
    """
    ...

def dispatch_reduction_ufunc(self, ufunc: np.ufunc, method: str, *inputs, **kwargs): # -> _NotImplementedType | Any:
    """
    Dispatch ufunc reductions to self's reduction methods.
    """
    ...




"""
This type stub file was generated by pyright.
"""

from typing import Iterable, Literal, Pattern, Sequence, TYPE_CHECKING
from pandas._typing import FilePath, ReadBuffer
from pandas.util._decorators import deprecate_nonkeyword_arguments
from pandas import DataFrame

"""
:mod:`pandas.io.html` is a module containing functionality for dealing with
HTML IO.

"""
if TYPE_CHECKING:
    ...
_IMPORTS = ...
_HAS_BS4 = ...
_HAS_LXML = ...
_HAS_HTML5LIB = ...
_RE_WHITESPACE = ...
class _HtmlFrameParser:
    """
    Base class for parsers that parse HTML into DataFrames.

    Parameters
    ----------
    io : str or file-like
        This can be either a string of raw HTML, a valid URL using the HTTP,
        FTP, or FILE protocols or a file-like object.

    match : str or regex
        The text to match in the document.

    attrs : dict
        List of HTML <table> element attributes to match.

    encoding : str
        Encoding to be used by parser

    displayed_only : bool
        Whether or not items with "display:none" should be ignored

    extract_links : {None, "all", "header", "body", "footer"}
        Table elements in the specified section(s) with <a> tags will have their
        href extracted.

        .. versionadded:: 1.5.0

    Attributes
    ----------
    io : str or file-like
        raw HTML, URL, or file-like object

    match : regex
        The text to match in the raw HTML

    attrs : dict-like
        A dictionary of valid table attributes to use to search for table
        elements.

    encoding : str
        Encoding to be used by parser

    displayed_only : bool
        Whether or not items with "display:none" should be ignored

    extract_links : {None, "all", "header", "body", "footer"}
        Table elements in the specified section(s) with <a> tags will have their
        href extracted.

        .. versionadded:: 1.5.0

    Notes
    -----
    To subclass this class effectively you must override the following methods:
        * :func:`_build_doc`
        * :func:`_attr_getter`
        * :func:`_href_getter`
        * :func:`_text_getter`
        * :func:`_parse_td`
        * :func:`_parse_thead_tr`
        * :func:`_parse_tbody_tr`
        * :func:`_parse_tfoot_tr`
        * :func:`_parse_tables`
        * :func:`_equals_tag`
    See each method's respective documentation for details on their
    functionality.
    """
    def __init__(self, io: FilePath | ReadBuffer[str] | ReadBuffer[bytes], match: str | Pattern, attrs: dict[str, str] | None, encoding: str, displayed_only: bool, extract_links: Literal[None, "header", "footer", "body", "all"]) -> None:
        ...
    
    def parse_tables(self):
        """
        Parse and return all tables from the DOM.

        Returns
        -------
        list of parsed (header, body, footer) tuples from tables.
        """
        ...
    


class _BeautifulSoupHtml5LibFrameParser(_HtmlFrameParser):
    """
    HTML to DataFrame parser that uses BeautifulSoup under the hood.

    See Also
    --------
    pandas.io.html._HtmlFrameParser
    pandas.io.html._LxmlFrameParser

    Notes
    -----
    Documentation strings for this class are in the base class
    :class:`pandas.io.html._HtmlFrameParser`.
    """
    def __init__(self, *args, **kwargs) -> None:
        ...
    


_re_namespace = ...
class _LxmlFrameParser(_HtmlFrameParser):
    """
    HTML to DataFrame parser that uses lxml under the hood.

    Warning
    -------
    This parser can only handle HTTP, FTP, and FILE urls.

    See Also
    --------
    _HtmlFrameParser
    _BeautifulSoupLxmlFrameParser

    Notes
    -----
    Documentation strings for this class are in the base class
    :class:`_HtmlFrameParser`.
    """
    ...


_valid_parsers = ...
@deprecate_nonkeyword_arguments(version="2.0")
def read_html(io: FilePath | ReadBuffer[str], match: str | Pattern = ..., flavor: str | None = ..., header: int | Sequence[int] | None = ..., index_col: int | Sequence[int] | None = ..., skiprows: int | Sequence[int] | slice | None = ..., attrs: dict[str, str] | None = ..., parse_dates: bool = ..., thousands: str | None = ..., encoding: str | None = ..., decimal: str = ..., converters: dict | None = ..., na_values: Iterable[object] | None = ..., keep_default_na: bool = ..., displayed_only: bool = ..., extract_links: Literal[None, "header", "footer", "body", "all"] = ...) -> list[DataFrame]:
    r"""
    Read HTML tables into a ``list`` of ``DataFrame`` objects.

    Parameters
    ----------
    io : str, path object, or file-like object
        String, path object (implementing ``os.PathLike[str]``), or file-like
        object implementing a string ``read()`` function.
        The string can represent a URL or the HTML itself. Note that
        lxml only accepts the http, ftp and file url protocols. If you have a
        URL that starts with ``'https'`` you might try removing the ``'s'``.

    match : str or compiled regular expression, optional
        The set of tables containing text matching this regex or string will be
        returned. Unless the HTML is extremely simple you will probably need to
        pass a non-empty string here. Defaults to '.+' (match any non-empty
        string). The default value will return all tables contained on a page.
        This value is converted to a regular expression so that there is
        consistent behavior between Beautiful Soup and lxml.

    flavor : str, optional
        The parsing engine to use. 'bs4' and 'html5lib' are synonymous with
        each other, they are both there for backwards compatibility. The
        default of ``None`` tries to use ``lxml`` to parse and if that fails it
        falls back on ``bs4`` + ``html5lib``.

    header : int or list-like, optional
        The row (or list of rows for a :class:`~pandas.MultiIndex`) to use to
        make the columns headers.

    index_col : int or list-like, optional
        The column (or list of columns) to use to create the index.

    skiprows : int, list-like or slice, optional
        Number of rows to skip after parsing the column integer. 0-based. If a
        sequence of integers or a slice is given, will skip the rows indexed by
        that sequence.  Note that a single element sequence means 'skip the nth
        row' whereas an integer means 'skip n rows'.

    attrs : dict, optional
        This is a dictionary of attributes that you can pass to use to identify
        the table in the HTML. These are not checked for validity before being
        passed to lxml or Beautiful Soup. However, these attributes must be
        valid HTML table attributes to work correctly. For example, ::

            attrs = {'id': 'table'}

        is a valid attribute dictionary because the 'id' HTML tag attribute is
        a valid HTML attribute for *any* HTML tag as per `this document
        <https://html.spec.whatwg.org/multipage/dom.html#global-attributes>`__. ::

            attrs = {'asdf': 'table'}

        is *not* a valid attribute dictionary because 'asdf' is not a valid
        HTML attribute even if it is a valid XML attribute.  Valid HTML 4.01
        table attributes can be found `here
        <http://www.w3.org/TR/REC-html40/struct/tables.html#h-11.2>`__. A
        working draft of the HTML 5 spec can be found `here
        <https://html.spec.whatwg.org/multipage/tables.html>`__. It contains the
        latest information on table attributes for the modern web.

    parse_dates : bool, optional
        See :func:`~read_csv` for more details.

    thousands : str, optional
        Separator to use to parse thousands. Defaults to ``','``.

    encoding : str, optional
        The encoding used to decode the web page. Defaults to ``None``.``None``
        preserves the previous encoding behavior, which depends on the
        underlying parser library (e.g., the parser library will try to use
        the encoding provided by the document).

    decimal : str, default '.'
        Character to recognize as decimal point (e.g. use ',' for European
        data).

    converters : dict, default None
        Dict of functions for converting values in certain columns. Keys can
        either be integers or column labels, values are functions that take one
        input argument, the cell (not column) content, and return the
        transformed content.

    na_values : iterable, default None
        Custom NA values.

    keep_default_na : bool, default True
        If na_values are specified and keep_default_na is False the default NaN
        values are overridden, otherwise they're appended to.

    displayed_only : bool, default True
        Whether elements with "display: none" should be parsed.

    extract_links : {None, "all", "header", "body", "footer"}
        Table elements in the specified section(s) with <a> tags will have their
        href extracted.

        .. versionadded:: 1.5.0

    Returns
    -------
    dfs
        A list of DataFrames.

    See Also
    --------
    read_csv : Read a comma-separated values (csv) file into DataFrame.

    Notes
    -----
    Before using this function you should read the :ref:`gotchas about the
    HTML parsing libraries <io.html.gotchas>`.

    Expect to do some cleanup after you call this function. For example, you
    might need to manually assign column names if the column names are
    converted to NaN when you pass the `header=0` argument. We try to assume as
    little as possible about the structure of the table and push the
    idiosyncrasies of the HTML contained in the table to the user.

    This function searches for ``<table>`` elements and only for ``<tr>``
    and ``<th>`` rows and ``<td>`` elements within each ``<tr>`` or ``<th>``
    element in the table. ``<td>`` stands for "table data". This function
    attempts to properly handle ``colspan`` and ``rowspan`` attributes.
    If the function has a ``<thead>`` argument, it is used to construct
    the header, otherwise the function attempts to find the header within
    the body (by putting rows with only ``<th>`` elements into the header).

    Similar to :func:`~read_csv` the `header` argument is applied
    **after** `skiprows` is applied.

    This function will *always* return a list of :class:`DataFrame` *or*
    it will fail, e.g., it will *not* return an empty list.

    Examples
    --------
    See the :ref:`read_html documentation in the IO section of the docs
    <io.read_html>` for some examples of reading in HTML tables.
    """
    ...




"""
This type stub file was generated by pyright.
"""

from typing import Literal, TYPE_CHECKING
from pandas._typing import PositionalIndexer
from pandas.util._decorators import doc
from pandas import DataFrame, Series
from pandas.core.groupby import groupby

if TYPE_CHECKING:
    ...
class GroupByIndexingMixin:
    """
    Mixin for adding ._positional_selector to GroupBy.
    """
    ...


@doc(GroupByIndexingMixin._positional_selector)
class GroupByPositionalSelector:
    def __init__(self, groupby_object: groupby.GroupBy) -> None:
        ...
    
    def __getitem__(self, arg: PositionalIndexer | tuple) -> DataFrame | Series:
        """
        Select by positional index per group.

        Implements GroupBy._positional_selector

        Parameters
        ----------
        arg : PositionalIndexer | tuple
            Allowed values are:
            - int
            - int valued iterable such as list or range
            - slice with step either None or positive
            - tuple of integers and slices

        Returns
        -------
        Series
            The filtered subset of the original groupby Series.
        DataFrame
            The filtered subset of the original groupby DataFrame.

        See Also
        --------
        DataFrame.iloc : Integer-location based indexing for selection by position.
        GroupBy.head : Return first n rows of each group.
        GroupBy.tail : Return last n rows of each group.
        GroupBy._positional_selector : Return positional selection for each group.
        GroupBy.nth : Take the nth row from each group if n is an int, or a
            subset of rows, if n is a list of ints.
        """
        ...
    


class GroupByNthSelector:
    """
    Dynamically substituted for GroupBy.nth to enable both call and index
    """
    def __init__(self, groupby_object: groupby.GroupBy) -> None:
        ...
    
    def __call__(self, n: PositionalIndexer | tuple, dropna: Literal["any", "all", None] = ...) -> DataFrame | Series:
        ...
    
    def __getitem__(self, n: PositionalIndexer | tuple) -> DataFrame | Series:
        ...
    





"""
This type stub file was generated by pyright.
"""

"""
This module is imported from the pandas package __init__.py file
in order to ensure that the core.config options registered here will
be available as soon as the user loads the package. if register_option
is invoked inside specific modules, they will not be registered until that
module is imported, which may or may not be a problem.

If you need to make sure options are available even before a certain
module is imported, register them here rather than in the module.

"""
use_bottleneck_doc = ...
def use_bottleneck_cb(key) -> None:
    ...

use_numexpr_doc = ...
def use_numexpr_cb(key) -> None:
    ...

use_numba_doc = ...
def use_numba_cb(key) -> None:
    ...

pc_precision_doc = ...
pc_colspace_doc = ...
pc_max_rows_doc = ...
pc_min_rows_doc = ...
pc_max_cols_doc = ...
pc_max_categories_doc = ...
pc_max_info_cols_doc = ...
pc_nb_repr_h_doc = ...
pc_pprint_nest_depth = ...
pc_multi_sparse_doc = ...
float_format_doc = ...
max_colwidth_doc = ...
colheader_justify_doc = ...
pc_expand_repr_doc = ...
pc_show_dimensions_doc = ...
pc_east_asian_width_doc = ...
pc_ambiguous_as_wide_doc = ...
pc_latex_repr_doc = ...
pc_table_schema_doc = ...
pc_html_border_doc = ...
pc_html_use_mathjax_doc = ...
pc_max_dir_items = ...
pc_width_doc = ...
pc_chop_threshold_doc = ...
pc_max_seq_items = ...
pc_max_info_rows_doc = ...
pc_large_repr_doc = ...
pc_memory_usage_doc = ...
pc_latex_escape = ...
pc_latex_longtable = ...
pc_latex_multicolumn = ...
pc_latex_multicolumn_format = ...
pc_latex_multirow = ...
def table_schema_cb(key) -> None:
    ...

def is_terminal() -> bool:
    """
    Detect if Python is running in a terminal.

    Returns True if Python is running in a terminal or False if not.
    """
    ...

tc_sim_interactive_doc = ...
use_inf_as_null_doc = ...
use_inf_as_na_doc = ...
def use_inf_as_na_cb(key) -> None:
    ...

data_manager_doc = ...
copy_on_write_doc = ...
chained_assignment = ...
string_storage_doc = ...
reader_engine_doc = ...
_xls_options = ...
_xlsm_options = ...
_xlsx_options = ...
_ods_options = ...
_xlsb_options = ...
writer_engine_doc = ...
_xls_options = ...
_xlsm_options = ...
_xlsx_options = ...
_ods_options = ...
parquet_engine_doc = ...
sql_engine_doc = ...
plotting_backend_doc = ...
def register_plotting_backend_cb(key) -> None:
    ...

register_converter_doc = ...
def register_converter_cb(key) -> None:
    ...

styler_sparse_index_doc = ...
styler_sparse_columns_doc = ...
styler_render_repr = ...
styler_max_elements = ...
styler_max_rows = ...
styler_max_columns = ...
styler_precision = ...
styler_decimal = ...
styler_thousands = ...
styler_na_rep = ...
styler_escape = ...
styler_formatter = ...
styler_multirow_align = ...
styler_multicol_align = ...
styler_hrules = ...
styler_environment = ...
styler_encoding = ...
styler_mathjax = ...



"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Any, Callable, Literal, Sequence, TYPE_CHECKING, TypeVar, Union, overload
from pandas._libs.tslibs import BaseOffset, NaTType
from pandas._typing import ArrayLike, DatetimeLikeScalar, Dtype, DtypeObj, NpDtype, PositionalIndexer2D, PositionalIndexerTuple, ScalarIndexer, SequenceIndexer, npt
from pandas.util._decorators import Appender, Substitution
from pandas.core.dtypes.dtypes import DatetimeTZDtype
from pandas.core.arraylike import OpsMixin
from pandas.core.arrays._mixins import NDArrayBackedExtensionArray, ravel_compat
from pandas.core.ops.common import unpack_zerodim_and_defer
from pandas.core.arrays import DatetimeArray, TimedeltaArray

if TYPE_CHECKING:
    ...
DTScalarOrNaT = Union[DatetimeLikeScalar, NaTType]
DatetimeLikeArrayT = TypeVar("DatetimeLikeArrayT", bound="DatetimeLikeArrayMixin")
class InvalidComparison(Exception):
    """
    Raised by _validate_comparison_value to indicate to caller it should
    return invalid_comparison.
    """
    ...


class DatetimeLikeArrayMixin(OpsMixin, NDArrayBackedExtensionArray):
    """
    Shared Base/Mixin class for DatetimeArray, TimedeltaArray, PeriodArray

    Assumes that __new__/__init__ defines:
        _data
        _freq

    and that the inheriting class has methods:
        _generate_range
    """
    _infer_matches: tuple[str, ...]
    _is_recognized_dtype: Callable[[DtypeObj], bool]
    _recognized_scalars: tuple[type, ...]
    _ndarray: np.ndarray
    def __init__(self, data, dtype: Dtype | None = ..., freq=..., copy=...) -> None:
        ...
    
    def __iter__(self): # -> Generator[DTScalarOrNaT, None, None]:
        ...
    
    @property
    def asi8(self) -> npt.NDArray[np.int64]:
        """
        Integer representation of the values.

        Returns
        -------
        ndarray
            An ndarray with int64 dtype.
        """
        ...
    
    def __array__(self, dtype: NpDtype | None = ...) -> np.ndarray:
        ...
    
    @overload
    def __getitem__(self, item: ScalarIndexer) -> DTScalarOrNaT:
        ...
    
    @overload
    def __getitem__(self: DatetimeLikeArrayT, item: SequenceIndexer | PositionalIndexerTuple) -> DatetimeLikeArrayT:
        ...
    
    def __getitem__(self: DatetimeLikeArrayT, key: PositionalIndexer2D) -> DatetimeLikeArrayT | DTScalarOrNaT:
        """
        This getitem defers to the underlying array, which by-definition can
        only handle list-likes, slices, and integer scalars
        """
        ...
    
    def __setitem__(self, key: int | Sequence[int] | Sequence[bool] | slice, value: NaTType | Any | Sequence[Any]) -> None:
        ...
    
    def astype(self, dtype, copy: bool = ...): # -> NDArray[object_] | ndarray[Any, Unknown] | ExtensionArray | NDArray[Any] | NDArray[int64] | NDArray[Unknown]:
        ...
    
    @overload
    def view(self: DatetimeLikeArrayT) -> DatetimeLikeArrayT:
        ...
    
    @overload
    def view(self, dtype: Literal["M8[ns]"]) -> DatetimeArray:
        ...
    
    @overload
    def view(self, dtype: Literal["m8[ns]"]) -> TimedeltaArray:
        ...
    
    @overload
    def view(self, dtype: Dtype | None = ...) -> ArrayLike:
        ...
    
    def view(self, dtype: Dtype | None = ...) -> ArrayLike:
        ...
    
    def copy(self: DatetimeLikeArrayT, order=...) -> DatetimeLikeArrayT:
        ...
    
    @ravel_compat
    def map(self, mapper): # -> ExtensionArray:
        ...
    
    def isin(self, values) -> npt.NDArray[np.bool_]:
        """
        Compute boolean array of whether each value is found in the
        passed set of values.

        Parameters
        ----------
        values : set or sequence of values

        Returns
        -------
        ndarray[bool]
        """
        ...
    
    def isna(self) -> npt.NDArray[np.bool_]:
        ...
    
    @property
    def freq(self): # -> None:
        """
        Return the frequency object if it is set, otherwise None.
        """
        ...
    
    @freq.setter
    def freq(self, value) -> None:
        ...
    
    @property
    def freqstr(self) -> str | None:
        """
        Return the frequency object as a string if its set, otherwise None.
        """
        ...
    
    @property
    def inferred_freq(self) -> str | None:
        """
        Tries to return a string representing a frequency generated by infer_freq.

        Returns None if it can't autodetect the frequency.
        """
        ...
    
    @property
    def resolution(self) -> str:
        """
        Returns day, hour, minute, second, millisecond or microsecond
        """
        ...
    
    __pow__ = ...
    __rpow__ = ...
    __mul__ = ...
    __rmul__ = ...
    __truediv__ = ...
    __rtruediv__ = ...
    __floordiv__ = ...
    __rfloordiv__ = ...
    __mod__ = ...
    __rmod__ = ...
    __divmod__ = ...
    __rdivmod__ = ...
    @unpack_zerodim_and_defer("__add__")
    def __add__(self, other): # -> DatetimeArray | _NotImplementedType | TimedeltaArray | PeriodArray | Self@DatetimeLikeArrayMixin | Any | ndarray[Any, Unknown]:
        ...
    
    def __radd__(self, other): # -> DatetimeArray | _NotImplementedType | TimedeltaArray | PeriodArray | Self@DatetimeLikeArrayMixin | Any | ndarray[Any, Unknown]:
        ...
    
    @unpack_zerodim_and_defer("__sub__")
    def __sub__(self, other): # -> _NotImplementedType | PeriodArray | Self@DatetimeLikeArrayMixin | Any | NDArray[Any] | NDArray[object_] | ndarray[Any, Unknown]:
        ...
    
    def __rsub__(self, other): # -> _NotImplementedType | PeriodArray | DatetimeArray | Any | NDArray[Any] | NDArray[object_] | ndarray[Any, Unknown] | DatetimeLikeArrayMixin:
        ...
    
    def __iadd__(self: DatetimeLikeArrayT, other) -> DatetimeLikeArrayT:
        ...
    
    def __isub__(self: DatetimeLikeArrayT, other) -> DatetimeLikeArrayT:
        ...
    
    def min(self, *, axis: int | None = ..., skipna: bool = ..., **kwargs): # -> NaTType:
        """
        Return the minimum value of the Array or minimum along
        an axis.

        See Also
        --------
        numpy.ndarray.min
        Index.min : Return the minimum value in an Index.
        Series.min : Return the minimum value in a Series.
        """
        ...
    
    def max(self, *, axis: int | None = ..., skipna: bool = ..., **kwargs): # -> Dtype | Type[str]:
        """
        Return the maximum value of the Array or maximum along
        an axis.

        See Also
        --------
        numpy.ndarray.max
        Index.max : Return the maximum value in an Index.
        Series.max : Return the maximum value in a Series.
        """
        ...
    
    def mean(self, *, skipna: bool = ..., axis: int | None = ...):
        """
        Return the mean value of the Array.

        .. versionadded:: 0.25.0

        Parameters
        ----------
        skipna : bool, default True
            Whether to ignore any NaT elements.
        axis : int, optional, default 0

        Returns
        -------
        scalar
            Timestamp or Timedelta.

        See Also
        --------
        numpy.ndarray.mean : Returns the average of array elements along a given axis.
        Series.mean : Return the mean value in a Series.

        Notes
        -----
        mean is only defined for Datetime and Timedelta dtypes, not for Period.
        """
        ...
    
    def median(self, *, axis: int | None = ..., skipna: bool = ..., **kwargs):
        ...
    


class DatelikeOps(DatetimeLikeArrayMixin):
    """
    Common ops for DatetimeIndex/PeriodIndex, but not TimedeltaIndex.
    """
    @Substitution(URL="https://docs.python.org/3/library/datetime.html" "#strftime-and-strptime-behavior")
    def strftime(self, date_format: str) -> npt.NDArray[np.object_]:
        """
        Convert to Index using specified date_format.

        Return an Index of formatted strings specified by date_format, which
        supports the same string format as the python standard library. Details
        of the string format can be found in `python string format
        doc <%(URL)s>`__.

        Formats supported by the C `strftime` API but not by the python string format
        doc (such as `"%%R"`, `"%%r"`) are not officially supported and should be
        preferably replaced with their supported equivalents (such as `"%%H:%%M"`,
        `"%%I:%%M:%%S %%p"`).

        Note that `PeriodIndex` support additional directives, detailed in
        `Period.strftime`.

        Parameters
        ----------
        date_format : str
            Date format string (e.g. "%%Y-%%m-%%d").

        Returns
        -------
        ndarray[object]
            NumPy ndarray of formatted strings.

        See Also
        --------
        to_datetime : Convert the given argument to datetime.
        DatetimeIndex.normalize : Return DatetimeIndex with times to midnight.
        DatetimeIndex.round : Round the DatetimeIndex to the specified freq.
        DatetimeIndex.floor : Floor the DatetimeIndex to the specified freq.
        Timestamp.strftime : Format a single Timestamp.
        Period.strftime : Format a single Period.

        Examples
        --------
        >>> rng = pd.date_range(pd.Timestamp("2018-03-10 09:00"),
        ...                     periods=3, freq='s')
        >>> rng.strftime('%%B %%d, %%Y, %%r')
        Index(['March 10, 2018, 09:00:00 AM', 'March 10, 2018, 09:00:01 AM',
               'March 10, 2018, 09:00:02 AM'],
              dtype='object')
        """
        ...
    


_round_doc = ...
_round_example = ...
_floor_example = ...
_ceil_example = ...
TimelikeOpsT = TypeVar("TimelikeOpsT", bound="TimelikeOps")
class TimelikeOps(DatetimeLikeArrayMixin):
    """
    Common ops for TimedeltaIndex/DatetimeIndex, but not PeriodIndex.
    """
    _default_dtype: np.dtype
    def __init__(self, values, dtype=..., freq=..., copy: bool = ...) -> None:
        ...
    
    def __array_ufunc__(self, ufunc: np.ufunc, method: str, *inputs, **kwargs): # -> Any | _NotImplementedType | tuple[Unknown, ...]:
        ...
    
    @Appender((_round_doc + _round_example).format(op="round"))
    def round(self, freq, ambiguous=..., nonexistent=...):
        ...
    
    @Appender((_round_doc + _floor_example).format(op="floor"))
    def floor(self, freq, ambiguous=..., nonexistent=...):
        ...
    
    @Appender((_round_doc + _ceil_example).format(op="ceil"))
    def ceil(self, freq, ambiguous=..., nonexistent=...):
        ...
    
    def any(self, *, axis: int | None = ..., skipna: bool = ...) -> bool:
        ...
    
    def all(self, *, axis: int | None = ..., skipna: bool = ...) -> bool:
        ...
    
    def factorize(self, na_sentinel: int = ..., sort: bool = ...): # -> tuple[ndarray[Any, dtype[intp]], TimelikeOps | Self@TimelikeOps] | tuple[ndarray[Unknown, Unknown], ExtensionArray]:
        ...
    


def ensure_arraylike_for_datetimelike(data, copy: bool, cls_name: str): # -> tuple[ArrayLike | NDArray[Unknown], bool]:
    ...

@overload
def validate_periods(periods: None) -> None:
    ...

@overload
def validate_periods(periods: float) -> int:
    ...

def validate_periods(periods: float | None) -> int | None:
    """
    If a `periods` argument is passed to the Datetime/Timedelta Array/Index
    constructor, cast it to an integer.

    Parameters
    ----------
    periods : None, float, int

    Returns
    -------
    periods : None or int

    Raises
    ------
    TypeError
        if periods is None, float, or int
    """
    ...

def validate_inferred_freq(freq, inferred_freq, freq_infer) -> tuple[BaseOffset | None, bool]:
    """
    If the user passes a freq and another freq is inferred from passed data,
    require that they match.

    Parameters
    ----------
    freq : DateOffset or None
    inferred_freq : DateOffset or None
    freq_infer : bool

    Returns
    -------
    freq : DateOffset or None
    freq_infer : bool

    Notes
    -----
    We assume at this point that `maybe_infer_freq` has been called, so
    `freq` is either a DateOffset object or None.
    """
    ...

def maybe_infer_freq(freq): # -> tuple[BaseOffset | None, bool]:
    """
    Comparing a DateOffset to the string "infer" raises, so we need to
    be careful about comparisons.  Make a dummy variable `freq_infer` to
    signify the case where the given freq is "infer" and set freq to None
    to avoid comparison trouble later on.

    Parameters
    ----------
    freq : {DateOffset, None, str}

    Returns
    -------
    freq : {DateOffset, None}
    freq_infer : bool
        Whether we should inherit the freq of passed data.
    """
    ...

def dtype_to_unit(dtype: DatetimeTZDtype | np.dtype) -> str:
    """
    Return the unit str corresponding to the dtype's resolution.

    Parameters
    ----------
    dtype : DatetimeTZDtype or np.dtype
        If np.dtype, we assume it is a datetime64 dtype.

    Returns
    -------
    str
    """
    ...




"""
This type stub file was generated by pyright.
"""

from pandas.io.excel._base import ExcelFile, ExcelWriter, read_excel
from pandas.io.excel._odswriter import ODSWriter as _ODSWriter
from pandas.io.excel._openpyxl import OpenpyxlWriter as _OpenpyxlWriter
from pandas.io.excel._util import register_writer
from pandas.io.excel._xlsxwriter import XlsxWriter as _XlsxWriter
from pandas.io.excel._xlwt import XlwtWriter as _XlwtWriter

__all__ = ["read_excel", "ExcelWriter", "ExcelFile"]



"""
This type stub file was generated by pyright.
"""

import numpy as np
from datetime import tzinfo
from typing import TYPE_CHECKING
from pandas._libs.tslibs import BaseOffset
from pandas._typing import npt
from pandas.core.dtypes.dtypes import DatetimeTZDtype
from pandas.core.arrays import PeriodArray, TimedeltaArray, datetimelike as dtl
from pandas import DataFrame

if TYPE_CHECKING:
    ...
_midnight = ...
def tz_to_dtype(tz: tzinfo | None, unit: str = ...): # -> dtype[Any] | DatetimeTZDtype:
    """
    Return a datetime64[ns] dtype appropriate for the given timezone.

    Parameters
    ----------
    tz : tzinfo or None
    unit : str, default "ns"

    Returns
    -------
    np.dtype or Datetime64TZDType
    """
    ...

class DatetimeArray(dtl.TimelikeOps, dtl.DatelikeOps):
    """
    Pandas ExtensionArray for tz-naive or tz-aware datetime data.

    .. warning::

       DatetimeArray is currently experimental, and its API may change
       without warning. In particular, :attr:`DatetimeArray.dtype` is
       expected to change to always be an instance of an ``ExtensionDtype``
       subclass.

    Parameters
    ----------
    values : Series, Index, DatetimeArray, ndarray
        The datetime data.

        For DatetimeArray `values` (or a Series or Index boxing one),
        `dtype` and `freq` will be extracted from `values`.

    dtype : numpy.dtype or DatetimeTZDtype
        Note that the only NumPy dtype allowed is 'datetime64[ns]'.
    freq : str or Offset, optional
        The frequency.
    copy : bool, default False
        Whether to copy the underlying array of values.

    Attributes
    ----------
    None

    Methods
    -------
    None
    """
    _typ = ...
    _internal_fill_value = ...
    _recognized_scalars = ...
    _is_recognized_dtype = ...
    _infer_matches = ...
    _bool_ops: list[str] = ...
    _object_ops: list[str] = ...
    _field_ops: list[str] = ...
    _other_ops: list[str] = ...
    _datetimelike_ops: list[str] = ...
    _datetimelike_methods: list[str] = ...
    __array_priority__ = ...
    _dtype: np.dtype | DatetimeTZDtype
    _freq: BaseOffset | None = ...
    _default_dtype = ...
    @property
    def dtype(self) -> np.dtype | DatetimeTZDtype:
        """
        The dtype for the DatetimeArray.

        .. warning::

           A future version of pandas will change dtype to never be a
           ``numpy.dtype``. Instead, :attr:`DatetimeArray.dtype` will
           always be an instance of an ``ExtensionDtype`` subclass.

        Returns
        -------
        numpy.dtype or DatetimeTZDtype
            If the values are tz-naive, then ``np.dtype('datetime64[ns]')``
            is returned.

            If the values are tz-aware, then the ``DatetimeTZDtype``
            is returned.
        """
        ...
    
    @property
    def tz(self) -> tzinfo | None:
        """
        Return the timezone.

        Returns
        -------
        datetime.tzinfo, pytz.tzinfo.BaseTZInfo, dateutil.tz.tz.tzfile, or None
            Returns None when the array is tz-naive.
        """
        ...
    
    @tz.setter
    def tz(self, value):
        ...
    
    @property
    def tzinfo(self) -> tzinfo | None:
        """
        Alias for tz attribute
        """
        ...
    
    @property
    def is_normalized(self) -> bool:
        """
        Returns True if all of the dates are at midnight ("no time")
        """
        ...
    
    def __array__(self, dtype=...) -> np.ndarray:
        ...
    
    def __iter__(self): # -> Generator[DTScalarOrNaT | Any, None, None]:
        """
        Return an iterator over the boxed values

        Yields
        ------
        tstamp : Timestamp
        """
        ...
    
    def astype(self, dtype, copy: bool = ...): # -> Self@DatetimeArray | DatetimeArray | PeriodArray | NDArray[object_] | ndarray[Any, Unknown] | ExtensionArray | NDArray[Any] | NDArray[int64] | NDArray[Unknown]:
        ...
    
    def tz_convert(self, tz) -> DatetimeArray:
        """
        Convert tz-aware Datetime Array/Index from one time zone to another.

        Parameters
        ----------
        tz : str, pytz.timezone, dateutil.tz.tzfile or None
            Time zone for time. Corresponding timestamps would be converted
            to this time zone of the Datetime Array/Index. A `tz` of None will
            convert to UTC and remove the timezone information.

        Returns
        -------
        Array or Index

        Raises
        ------
        TypeError
            If Datetime Array/Index is tz-naive.

        See Also
        --------
        DatetimeIndex.tz : A timezone that has a variable offset from UTC.
        DatetimeIndex.tz_localize : Localize tz-naive DatetimeIndex to a
            given time zone, or remove timezone from a tz-aware DatetimeIndex.

        Examples
        --------
        With the `tz` parameter, we can change the DatetimeIndex
        to other time zones:

        >>> dti = pd.date_range(start='2014-08-01 09:00',
        ...                     freq='H', periods=3, tz='Europe/Berlin')

        >>> dti
        DatetimeIndex(['2014-08-01 09:00:00+02:00',
                       '2014-08-01 10:00:00+02:00',
                       '2014-08-01 11:00:00+02:00'],
                      dtype='datetime64[ns, Europe/Berlin]', freq='H')

        >>> dti.tz_convert('US/Central')
        DatetimeIndex(['2014-08-01 02:00:00-05:00',
                       '2014-08-01 03:00:00-05:00',
                       '2014-08-01 04:00:00-05:00'],
                      dtype='datetime64[ns, US/Central]', freq='H')

        With the ``tz=None``, we can remove the timezone (after converting
        to UTC if necessary):

        >>> dti = pd.date_range(start='2014-08-01 09:00', freq='H',
        ...                     periods=3, tz='Europe/Berlin')

        >>> dti
        DatetimeIndex(['2014-08-01 09:00:00+02:00',
                       '2014-08-01 10:00:00+02:00',
                       '2014-08-01 11:00:00+02:00'],
                        dtype='datetime64[ns, Europe/Berlin]', freq='H')

        >>> dti.tz_convert(None)
        DatetimeIndex(['2014-08-01 07:00:00',
                       '2014-08-01 08:00:00',
                       '2014-08-01 09:00:00'],
                        dtype='datetime64[ns]', freq='H')
        """
        ...
    
    @dtl.ravel_compat
    def tz_localize(self, tz, ambiguous=..., nonexistent=...) -> DatetimeArray:
        """
        Localize tz-naive Datetime Array/Index to tz-aware Datetime Array/Index.

        This method takes a time zone (tz) naive Datetime Array/Index object
        and makes this time zone aware. It does not move the time to another
        time zone.

        This method can also be used to do the inverse -- to create a time
        zone unaware object from an aware object. To that end, pass `tz=None`.

        Parameters
        ----------
        tz : str, pytz.timezone, dateutil.tz.tzfile or None
            Time zone to convert timestamps to. Passing ``None`` will
            remove the time zone information preserving local time.
        ambiguous : 'infer', 'NaT', bool array, default 'raise'
            When clocks moved backward due to DST, ambiguous times may arise.
            For example in Central European Time (UTC+01), when going from
            03:00 DST to 02:00 non-DST, 02:30:00 local time occurs both at
            00:30:00 UTC and at 01:30:00 UTC. In such a situation, the
            `ambiguous` parameter dictates how ambiguous times should be
            handled.

            - 'infer' will attempt to infer fall dst-transition hours based on
              order
            - bool-ndarray where True signifies a DST time, False signifies a
              non-DST time (note that this flag is only applicable for
              ambiguous times)
            - 'NaT' will return NaT where there are ambiguous times
            - 'raise' will raise an AmbiguousTimeError if there are ambiguous
              times.

        nonexistent : 'shift_forward', 'shift_backward, 'NaT', timedelta, \
default 'raise'
            A nonexistent time does not exist in a particular timezone
            where clocks moved forward due to DST.

            - 'shift_forward' will shift the nonexistent time forward to the
              closest existing time
            - 'shift_backward' will shift the nonexistent time backward to the
              closest existing time
            - 'NaT' will return NaT where there are nonexistent times
            - timedelta objects will shift nonexistent times by the timedelta
            - 'raise' will raise an NonExistentTimeError if there are
              nonexistent times.

        Returns
        -------
        Same type as self
            Array/Index converted to the specified time zone.

        Raises
        ------
        TypeError
            If the Datetime Array/Index is tz-aware and tz is not None.

        See Also
        --------
        DatetimeIndex.tz_convert : Convert tz-aware DatetimeIndex from
            one time zone to another.

        Examples
        --------
        >>> tz_naive = pd.date_range('2018-03-01 09:00', periods=3)
        >>> tz_naive
        DatetimeIndex(['2018-03-01 09:00:00', '2018-03-02 09:00:00',
                       '2018-03-03 09:00:00'],
                      dtype='datetime64[ns]', freq='D')

        Localize DatetimeIndex in US/Eastern time zone:

        >>> tz_aware = tz_naive.tz_localize(tz='US/Eastern')
        >>> tz_aware
        DatetimeIndex(['2018-03-01 09:00:00-05:00',
                       '2018-03-02 09:00:00-05:00',
                       '2018-03-03 09:00:00-05:00'],
                      dtype='datetime64[ns, US/Eastern]', freq=None)

        With the ``tz=None``, we can remove the time zone information
        while keeping the local time (not converted to UTC):

        >>> tz_aware.tz_localize(None)
        DatetimeIndex(['2018-03-01 09:00:00', '2018-03-02 09:00:00',
                       '2018-03-03 09:00:00'],
                      dtype='datetime64[ns]', freq=None)

        Be careful with DST changes. When there is sequential data, pandas can
        infer the DST time:

        >>> s = pd.to_datetime(pd.Series(['2018-10-28 01:30:00',
        ...                               '2018-10-28 02:00:00',
        ...                               '2018-10-28 02:30:00',
        ...                               '2018-10-28 02:00:00',
        ...                               '2018-10-28 02:30:00',
        ...                               '2018-10-28 03:00:00',
        ...                               '2018-10-28 03:30:00']))
        >>> s.dt.tz_localize('CET', ambiguous='infer')
        0   2018-10-28 01:30:00+02:00
        1   2018-10-28 02:00:00+02:00
        2   2018-10-28 02:30:00+02:00
        3   2018-10-28 02:00:00+01:00
        4   2018-10-28 02:30:00+01:00
        5   2018-10-28 03:00:00+01:00
        6   2018-10-28 03:30:00+01:00
        dtype: datetime64[ns, CET]

        In some cases, inferring the DST is impossible. In such cases, you can
        pass an ndarray to the ambiguous parameter to set the DST explicitly

        >>> s = pd.to_datetime(pd.Series(['2018-10-28 01:20:00',
        ...                               '2018-10-28 02:36:00',
        ...                               '2018-10-28 03:46:00']))
        >>> s.dt.tz_localize('CET', ambiguous=np.array([True, True, False]))
        0   2018-10-28 01:20:00+02:00
        1   2018-10-28 02:36:00+02:00
        2   2018-10-28 03:46:00+01:00
        dtype: datetime64[ns, CET]

        If the DST transition causes nonexistent times, you can shift these
        dates forward or backwards with a timedelta object or `'shift_forward'`
        or `'shift_backwards'`.

        >>> s = pd.to_datetime(pd.Series(['2015-03-29 02:30:00',
        ...                               '2015-03-29 03:30:00']))
        >>> s.dt.tz_localize('Europe/Warsaw', nonexistent='shift_forward')
        0   2015-03-29 03:00:00+02:00
        1   2015-03-29 03:30:00+02:00
        dtype: datetime64[ns, Europe/Warsaw]

        >>> s.dt.tz_localize('Europe/Warsaw', nonexistent='shift_backward')
        0   2015-03-29 01:59:59.999999999+01:00
        1   2015-03-29 03:30:00+02:00
        dtype: datetime64[ns, Europe/Warsaw]

        >>> s.dt.tz_localize('Europe/Warsaw', nonexistent=pd.Timedelta('1H'))
        0   2015-03-29 03:30:00+02:00
        1   2015-03-29 03:30:00+02:00
        dtype: datetime64[ns, Europe/Warsaw]
        """
        ...
    
    def to_pydatetime(self) -> npt.NDArray[np.object_]:
        """
        Return an ndarray of datetime.datetime objects.

        Returns
        -------
        datetimes : ndarray[object]
        """
        ...
    
    def normalize(self) -> DatetimeArray:
        """
        Convert times to midnight.

        The time component of the date-time is converted to midnight i.e.
        00:00:00. This is useful in cases, when the time does not matter.
        Length is unaltered. The timezones are unaffected.

        This method is available on Series with datetime values under
        the ``.dt`` accessor, and directly on Datetime Array/Index.

        Returns
        -------
        DatetimeArray, DatetimeIndex or Series
            The same type as the original data. Series will have the same
            name and index. DatetimeIndex will have the same name.

        See Also
        --------
        floor : Floor the datetimes to the specified freq.
        ceil : Ceil the datetimes to the specified freq.
        round : Round the datetimes to the specified freq.

        Examples
        --------
        >>> idx = pd.date_range(start='2014-08-01 10:00', freq='H',
        ...                     periods=3, tz='Asia/Calcutta')
        >>> idx
        DatetimeIndex(['2014-08-01 10:00:00+05:30',
                       '2014-08-01 11:00:00+05:30',
                       '2014-08-01 12:00:00+05:30'],
                        dtype='datetime64[ns, Asia/Calcutta]', freq='H')
        >>> idx.normalize()
        DatetimeIndex(['2014-08-01 00:00:00+05:30',
                       '2014-08-01 00:00:00+05:30',
                       '2014-08-01 00:00:00+05:30'],
                       dtype='datetime64[ns, Asia/Calcutta]', freq=None)
        """
        ...
    
    def to_period(self, freq=...) -> PeriodArray:
        """
        Cast to PeriodArray/Index at a particular frequency.

        Converts DatetimeArray/Index to PeriodArray/Index.

        Parameters
        ----------
        freq : str or Offset, optional
            One of pandas' :ref:`offset strings <timeseries.offset_aliases>`
            or an Offset object. Will be inferred by default.

        Returns
        -------
        PeriodArray/Index

        Raises
        ------
        ValueError
            When converting a DatetimeArray/Index with non-regular values,
            so that a frequency cannot be inferred.

        See Also
        --------
        PeriodIndex: Immutable ndarray holding ordinal values.
        DatetimeIndex.to_pydatetime: Return DatetimeIndex as object.

        Examples
        --------
        >>> df = pd.DataFrame({"y": [1, 2, 3]},
        ...                   index=pd.to_datetime(["2000-03-31 00:00:00",
        ...                                         "2000-05-31 00:00:00",
        ...                                         "2000-08-31 00:00:00"]))
        >>> df.index.to_period("M")
        PeriodIndex(['2000-03', '2000-05', '2000-08'],
                    dtype='period[M]')

        Infer the daily frequency

        >>> idx = pd.date_range("2017-01-01", periods=2)
        >>> idx.to_period()
        PeriodIndex(['2017-01-01', '2017-01-02'],
                    dtype='period[D]')
        """
        ...
    
    def to_perioddelta(self, freq) -> TimedeltaArray:
        """
        Calculate deltas between self values and self converted to Periods at a freq.

        Used for vectorized offsets.

        Parameters
        ----------
        freq : Period frequency

        Returns
        -------
        TimedeltaArray/Index
        """
        ...
    
    def month_name(self, locale=...) -> npt.NDArray[np.object_]:
        """
        Return the month names with specified locale.

        Parameters
        ----------
        locale : str, optional
            Locale determining the language in which to return the month name.
            Default is English locale.

        Returns
        -------
        Series or Index
            Series or Index of month names.

        Examples
        --------
        >>> s = pd.Series(pd.date_range(start='2018-01', freq='M', periods=3))
        >>> s
        0   2018-01-31
        1   2018-02-28
        2   2018-03-31
        dtype: datetime64[ns]
        >>> s.dt.month_name()
        0     January
        1    February
        2       March
        dtype: object

        >>> idx = pd.date_range(start='2018-01', freq='M', periods=3)
        >>> idx
        DatetimeIndex(['2018-01-31', '2018-02-28', '2018-03-31'],
                      dtype='datetime64[ns]', freq='M')
        >>> idx.month_name()
        Index(['January', 'February', 'March'], dtype='object')
        """
        ...
    
    def day_name(self, locale=...) -> npt.NDArray[np.object_]:
        """
        Return the day names with specified locale.

        Parameters
        ----------
        locale : str, optional
            Locale determining the language in which to return the day name.
            Default is English locale.

        Returns
        -------
        Series or Index
            Series or Index of day names.

        Examples
        --------
        >>> s = pd.Series(pd.date_range(start='2018-01-01', freq='D', periods=3))
        >>> s
        0   2018-01-01
        1   2018-01-02
        2   2018-01-03
        dtype: datetime64[ns]
        >>> s.dt.day_name()
        0       Monday
        1      Tuesday
        2    Wednesday
        dtype: object

        >>> idx = pd.date_range(start='2018-01-01', freq='D', periods=3)
        >>> idx
        DatetimeIndex(['2018-01-01', '2018-01-02', '2018-01-03'],
                      dtype='datetime64[ns]', freq='D')
        >>> idx.day_name()
        Index(['Monday', 'Tuesday', 'Wednesday'], dtype='object')
        """
        ...
    
    @property
    def time(self) -> npt.NDArray[np.object_]:
        """
        Returns numpy array of :class:`datetime.time` objects.

        The time part of the Timestamps.
        """
        ...
    
    @property
    def timetz(self) -> npt.NDArray[np.object_]:
        """
        Returns numpy array of :class:`datetime.time` objects with timezones.

        The time part of the Timestamps.
        """
        ...
    
    @property
    def date(self) -> npt.NDArray[np.object_]:
        """
        Returns numpy array of python :class:`datetime.date` objects.

        Namely, the date part of Timestamps without time and
        timezone information.
        """
        ...
    
    def isocalendar(self) -> DataFrame:
        """
        Calculate year, week, and day according to the ISO 8601 standard.

        .. versionadded:: 1.1.0

        Returns
        -------
        DataFrame
            With columns year, week and day.

        See Also
        --------
        Timestamp.isocalendar : Function return a 3-tuple containing ISO year,
            week number, and weekday for the given Timestamp object.
        datetime.date.isocalendar : Return a named tuple object with
            three components: year, week and weekday.

        Examples
        --------
        >>> idx = pd.date_range(start='2019-12-29', freq='D', periods=4)
        >>> idx.isocalendar()
                    year  week  day
        2019-12-29  2019    52    7
        2019-12-30  2020     1    1
        2019-12-31  2020     1    2
        2020-01-01  2020     1    3
        >>> idx.isocalendar().week
        2019-12-29    52
        2019-12-30     1
        2019-12-31     1
        2020-01-01     1
        Freq: D, Name: week, dtype: UInt32
        """
        ...
    
    @property
    def weekofyear(self): # -> Any:
        """
        The week ordinal of the year.

        .. deprecated:: 1.1.0

        weekofyear and week have been deprecated.
        Please use DatetimeIndex.isocalendar().week instead.
        """
        ...
    
    week = ...
    year = ...
    month = ...
    day = ...
    hour = ...
    minute = ...
    second = ...
    microsecond = ...
    nanosecond = ...
    _dayofweek_doc = ...
    day_of_week = ...
    dayofweek = ...
    weekday = ...
    day_of_year = ...
    dayofyear = ...
    quarter = ...
    days_in_month = ...
    daysinmonth = ...
    _is_month_doc = ...
    is_month_start = ...
    is_month_end = ...
    is_quarter_start = ...
    is_quarter_end = ...
    is_year_start = ...
    is_year_end = ...
    is_leap_year = ...
    def to_julian_date(self) -> npt.NDArray[np.float64]:
        """
        Convert Datetime Array to float64 ndarray of Julian Dates.
        0 Julian date is noon January 1, 4713 BC.
        https://en.wikipedia.org/wiki/Julian_day
        """
        ...
    
    def std(self, axis=..., dtype=..., out=..., ddof: int = ..., keepdims: bool = ..., skipna: bool = ...): # -> Timedelta | NaTType:
        """
        Return sample standard deviation over requested axis.

        Normalized by N-1 by default. This can be changed using the ddof argument

        Parameters
        ----------
        axis : int optional, default None
            Axis for the function to be applied on.
            For `Series` this parameter is unused and defaults to `None`.
        ddof : int, default 1
            Degrees of Freedom. The divisor used in calculations is N - ddof,
            where N represents the number of elements.
        skipna : bool, default True
            Exclude NA/null values. If an entire row/column is NA, the result will be
            NA.

        Returns
        -------
        Timedelta
        """
        ...
    


def sequence_to_datetimes(data, require_iso8601: bool = ...) -> DatetimeArray:
    """
    Parse/convert the passed data to either DatetimeArray or np.ndarray[object].
    """
    ...

def objects_to_datetime64ns(data: np.ndarray, dayfirst, yearfirst, utc=..., errors=..., require_iso8601: bool = ..., allow_object: bool = ..., allow_mixed: bool = ...): # -> tuple[NDArray[Any], tzinfo] | tuple[ndarray[Any, Unknown], None]:
    """
    Convert data to array of timestamps.

    Parameters
    ----------
    data : np.ndarray[object]
    dayfirst : bool
    yearfirst : bool
    utc : bool, default False
        Whether to convert timezone-aware timestamps to UTC.
    errors : {'raise', 'ignore', 'coerce'}
    require_iso8601 : bool, default False
    allow_object : bool
        Whether to return an object-dtype ndarray instead of raising if the
        data contains more than one timezone.
    allow_mixed : bool, default False
        Interpret integers as timestamps when datetime objects are also present.

    Returns
    -------
    result : ndarray
        np.int64 dtype if returned values represent UTC timestamps
        np.datetime64[ns] if returned values represent wall times
        object if mixed timezones
    inferred_tz : tzinfo or None

    Raises
    ------
    ValueError : if data cannot be converted to datetimes
    """
    ...

def maybe_convert_dtype(data, copy: bool, tz: tzinfo | None = ...): # -> tuple[Unknown, bool] | tuple[Unknown | NDArray[object_], bool]:
    """
    Convert data based on dtype conventions, issuing deprecation warnings
    or errors where appropriate.

    Parameters
    ----------
    data : np.ndarray or pd.Index
    copy : bool
    tz : tzinfo or None, default None

    Returns
    -------
    data : np.ndarray or pd.Index
    copy : bool

    Raises
    ------
    TypeError : PeriodDType data is passed
    """
    ...

def validate_tz_from_dtype(dtype, tz: tzinfo | None) -> tzinfo | None:
    """
    If the given dtype is a DatetimeTZDtype, extract the implied
    tzinfo object from it and check that it does not conflict with the given
    tz.

    Parameters
    ----------
    dtype : dtype, str
    tz : None, tzinfo

    Returns
    -------
    tz : consensus tzinfo

    Raises
    ------
    ValueError : on tzinfo mismatch
    """
    ...

def generate_range(start=..., end=..., periods=..., offset=...): # -> Generator[Unknown | datetime | Timestamp, None, None]:
    """
    Generates a sequence of dates corresponding to the specified time
    offset. Similar to dateutil.rrule except uses pandas DateOffset
    objects to represent time increments.

    Parameters
    ----------
    start : datetime, (default None)
    end : datetime, (default None)
    periods : int, (default None)
    offset : DateOffset, (default BDay())

    Notes
    -----
    * This method is faster for generating weekdays than dateutil.rrule
    * At least two of (start, end, periods) must be specified.
    * If both start and end are specified, the returned dates will
    satisfy start <= date <= end.

    Returns
    -------
    dates : generator object
    """
    ...




"""
This type stub file was generated by pyright.
"""

from typing import Any, Callable, Hashable, Literal, MutableMapping, Sequence, TYPE_CHECKING, TypeVar, overload
from pandas.io.excel._base import ExcelWriter

if TYPE_CHECKING:
    ExcelWriter_t = type[ExcelWriter]
    usecols_func = TypeVar("usecols_func", bound=Callable[[Hashable], object])
_writers: MutableMapping[str, ExcelWriter_t] = ...
def register_writer(klass: ExcelWriter_t) -> None:
    """
    Add engine to the excel writer registry.io.excel.

    You must use this method to integrate with ``to_excel``.

    Parameters
    ----------
    klass : ExcelWriter
    """
    ...

def get_default_engine(ext: str, mode: Literal["reader", "writer"] = ...) -> str:
    """
    Return the default reader/writer for the given extension.

    Parameters
    ----------
    ext : str
        The excel file extension for which to get the default engine.
    mode : str {'reader', 'writer'}
        Whether to get the default engine for reading or writing.
        Either 'reader' or 'writer'

    Returns
    -------
    str
        The default engine for the extension.
    """
    ...

def get_writer(engine_name: str) -> ExcelWriter_t:
    ...

@overload
def maybe_convert_usecols(usecols: str | list[int]) -> list[int]:
    ...

@overload
def maybe_convert_usecols(usecols: list[str]) -> list[str]:
    ...

@overload
def maybe_convert_usecols(usecols: usecols_func) -> usecols_func:
    ...

@overload
def maybe_convert_usecols(usecols: None) -> None:
    ...

def maybe_convert_usecols(usecols: str | list[int] | list[str] | usecols_func | None) -> None | list[int] | list[str] | usecols_func:
    """
    Convert `usecols` into a compatible format for parsing in `parsers.py`.

    Parameters
    ----------
    usecols : object
        The use-columns object to potentially convert.

    Returns
    -------
    converted : object
        The compatible format of `usecols`.
    """
    ...

@overload
def validate_freeze_panes(freeze_panes: tuple[int, int]) -> Literal[True]:
    ...

@overload
def validate_freeze_panes(freeze_panes: None) -> Literal[False]:
    ...

def validate_freeze_panes(freeze_panes: tuple[int, int] | None) -> bool:
    ...

def fill_mi_header(row: list[Hashable], control_row: list[bool]) -> tuple[list[Hashable], list[bool]]:
    """
    Forward fill blank entries in row but only inside the same parent index.

    Used for creating headers in Multiindex.

    Parameters
    ----------
    row : list
        List of items in a single row.
    control_row : list of bool
        Helps to determine if particular column is in same parent index as the
        previous value. Used to stop propagation of empty cells between
        different indexes.

    Returns
    -------
    Returns changed row and control_row
    """
    ...

def pop_header_name(row: list[Hashable], index_col: int | Sequence[int]) -> tuple[Hashable | None, list[Hashable]]:
    """
    Pop the header name for MultiIndex parsing.

    Parameters
    ----------
    row : list
        The data row to parse for the header name.
    index_col : int, list
        The index columns for our data. Assumed to be non-null.

    Returns
    -------
    header_name : str
        The extracted header name.
    trimmed_row : list
        The original data row with the header name removed.
    """
    ...

def combine_kwargs(engine_kwargs: dict[str, Any] | None, kwargs: dict) -> dict:
    """
    Used to combine two sources of kwargs for the backend engine.

    Use of kwargs is deprecated, this function is solely for use in 1.3 and should
    be removed in 1.4/2.0. Also _base.ExcelWriter.__new__ ensures either engine_kwargs
    or kwargs must be None or empty respectively.

    Parameters
    ----------
    engine_kwargs: dict
        kwargs to be passed through to the engine.
    kwargs: dict
        kwargs to be psased through to the engine (deprecated)

    Returns
    -------
    engine_kwargs combined with kwargs
    """
    ...




"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING
from pandas.core.computation.ops import BinOp

"""
Top level ``eval`` module.
"""
if TYPE_CHECKING:
    ...
def eval(expr: str | BinOp, parser: str = ..., engine: str | None = ..., truediv=..., local_dict=..., global_dict=..., resolvers=..., level=..., target=..., inplace=...): # -> NDFrame | object | None:
    """
    Evaluate a Python expression as a string using various backends.

    The following arithmetic operations are supported: ``+``, ``-``, ``*``,
    ``/``, ``**``, ``%``, ``//`` (python engine only) along with the following
    boolean operations: ``|`` (or), ``&`` (and), and ``~`` (not).
    Additionally, the ``'pandas'`` parser allows the use of :keyword:`and`,
    :keyword:`or`, and :keyword:`not` with the same semantics as the
    corresponding bitwise operators.  :class:`~pandas.Series` and
    :class:`~pandas.DataFrame` objects are supported and behave as they would
    with plain ol' Python evaluation.

    Parameters
    ----------
    expr : str
        The expression to evaluate. This string cannot contain any Python
        `statements
        <https://docs.python.org/3/reference/simple_stmts.html#simple-statements>`__,
        only Python `expressions
        <https://docs.python.org/3/reference/simple_stmts.html#expression-statements>`__.
    parser : {'pandas', 'python'}, default 'pandas'
        The parser to use to construct the syntax tree from the expression. The
        default of ``'pandas'`` parses code slightly different than standard
        Python. Alternatively, you can parse an expression using the
        ``'python'`` parser to retain strict Python semantics.  See the
        :ref:`enhancing performance <enhancingperf.eval>` documentation for
        more details.
    engine : {'python', 'numexpr'}, default 'numexpr'

        The engine used to evaluate the expression. Supported engines are

        - None : tries to use ``numexpr``, falls back to ``python``
        - ``'numexpr'`` : This default engine evaluates pandas objects using
          numexpr for large speed ups in complex expressions with large frames.
        - ``'python'`` : Performs operations as if you had ``eval``'d in top
          level python. This engine is generally not that useful.

        More backends may be available in the future.

    truediv : bool, optional
        Whether to use true division, like in Python >= 3.

        .. deprecated:: 1.0.0

    local_dict : dict or None, optional
        A dictionary of local variables, taken from locals() by default.
    global_dict : dict or None, optional
        A dictionary of global variables, taken from globals() by default.
    resolvers : list of dict-like or None, optional
        A list of objects implementing the ``__getitem__`` special method that
        you can use to inject an additional collection of namespaces to use for
        variable lookup. For example, this is used in the
        :meth:`~DataFrame.query` method to inject the
        ``DataFrame.index`` and ``DataFrame.columns``
        variables that refer to their respective :class:`~pandas.DataFrame`
        instance attributes.
    level : int, optional
        The number of prior stack frames to traverse and add to the current
        scope. Most users will **not** need to change this parameter.
    target : object, optional, default None
        This is the target object for assignment. It is used when there is
        variable assignment in the expression. If so, then `target` must
        support item assignment with string keys, and if a copy is being
        returned, it must also support `.copy()`.
    inplace : bool, default False
        If `target` is provided, and the expression mutates `target`, whether
        to modify `target` inplace. Otherwise, return a copy of `target` with
        the mutation.

    Returns
    -------
    ndarray, numeric scalar, DataFrame, Series, or None
        The completion value of evaluating the given code or None if ``inplace=True``.

    Raises
    ------
    ValueError
        There are many instances where such an error can be raised:

        - `target=None`, but the expression is multiline.
        - The expression is multiline, but not all them have item assignment.
          An example of such an arrangement is this:

          a = b + 1
          a + 2

          Here, there are expressions on different lines, making it multiline,
          but the last line has no variable assigned to the output of `a + 2`.
        - `inplace=True`, but the expression is missing item assignment.
        - Item assignment is provided, but the `target` does not support
          string item assignment.
        - Item assignment is provided and `inplace=False`, but the `target`
          does not support the `.copy()` method

    See Also
    --------
    DataFrame.query : Evaluates a boolean expression to query the columns
            of a frame.
    DataFrame.eval : Evaluate a string describing operations on
            DataFrame columns.

    Notes
    -----
    The ``dtype`` of any objects involved in an arithmetic ``%`` operation are
    recursively cast to ``float64``.

    See the :ref:`enhancing performance <enhancingperf.eval>` documentation for
    more details.

    Examples
    --------
    >>> df = pd.DataFrame({"animal": ["dog", "pig"], "age": [10, 20]})
    >>> df
      animal  age
    0    dog   10
    1    pig   20

    We can add a new column using ``pd.eval``:

    >>> pd.eval("double_age = df.age * 2", target=df)
      animal  age  double_age
    0    dog   10          20
    1    pig   20          40
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas._typing import npt

def build_field_sarray(dtindex: npt.NDArray[np.int64], reso: int) -> np.ndarray:
    ...

def month_position_check(fields, weekdays) -> str | None:
    ...

def get_date_name_field(dtindex: npt.NDArray[np.int64], field: str, locale: str | None = ..., reso: int = ...) -> npt.NDArray[np.object_]:
    ...

def get_start_end_field(dtindex: npt.NDArray[np.int64], field: str, freqstr: str | None = ..., month_kw: int = ..., reso: int = ...) -> npt.NDArray[np.bool_]:
    ...

def get_date_field(dtindex: npt.NDArray[np.int64], field: str, reso: int = ...) -> npt.NDArray[np.int32]:
    ...

def get_timedelta_field(tdindex: npt.NDArray[np.int64], field: str, reso: int = ...) -> npt.NDArray[np.int32]:
    ...

def isleapyear_arr(years: np.ndarray) -> npt.NDArray[np.bool_]:
    ...

def build_isocalendar_sarray(dtindex: npt.NDArray[np.int64], reso: int) -> np.ndarray:
    ...

class RoundTo:
    @property
    def MINUS_INFTY(self) -> int:
        ...
    
    @property
    def PLUS_INFTY(self) -> int:
        ...
    
    @property
    def NEAREST_HALF_EVEN(self) -> int:
        ...
    
    @property
    def NEAREST_HALF_PLUS_INFTY(self) -> int:
        ...
    
    @property
    def NEAREST_HALF_MINUS_INFTY(self) -> int:
        ...
    


def round_nsint64(values: npt.NDArray[np.int64], mode: RoundTo, nanos: int) -> npt.NDArray[np.int64]:
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Any, Sized, TYPE_CHECKING, TypeVar, overload
from pandas._typing import ArrayLike, Dtype, DtypeObj, Scalar
from pandas.core.dtypes.dtypes import ExtensionDtype
from pandas import Index
from pandas.core.arrays import DatetimeArray, ExtensionArray, IntervalArray, PeriodArray, TimedeltaArray

"""
Routines for casting.
"""
if TYPE_CHECKING:
    ...
_int8_max = ...
_int16_max = ...
_int32_max = ...
_int64_max = ...
_dtype_obj = ...
NumpyArrayT = TypeVar("NumpyArrayT", bound=np.ndarray)
def maybe_convert_platform(values: list | tuple | range | np.ndarray | ExtensionArray) -> ArrayLike:
    """try to do platform conversion, allow ndarray or list here"""
    ...

def is_nested_object(obj) -> bool:
    """
    return a boolean if we have a nested object, e.g. a Series with 1 or
    more Series elements

    This may not be necessarily be performant.

    """
    ...

def maybe_box_datetimelike(value: Scalar, dtype: Dtype | None = ...) -> Scalar:
    """
    Cast scalar to Timestamp or Timedelta if scalar is datetime-like
    and dtype is not object.

    Parameters
    ----------
    value : scalar
    dtype : Dtype, optional

    Returns
    -------
    scalar
    """
    ...

def maybe_box_native(value: Scalar) -> Scalar:
    """
    If passed a scalar cast the scalar to a python native type.

    Parameters
    ----------
    value : scalar or Series

    Returns
    -------
    scalar or Series
    """
    ...

@overload
def maybe_downcast_to_dtype(result: np.ndarray, dtype: str | np.dtype) -> np.ndarray:
    ...

@overload
def maybe_downcast_to_dtype(result: ExtensionArray, dtype: str | np.dtype) -> ArrayLike:
    ...

def maybe_downcast_to_dtype(result: ArrayLike, dtype: str | np.dtype) -> ArrayLike:
    """
    try to cast to the specified dtype (e.g. convert back to bool/int
    or could be an astype of float64->float32
    """
    ...

@overload
def maybe_downcast_numeric(result: np.ndarray, dtype: np.dtype, do_round: bool = ...) -> np.ndarray:
    ...

@overload
def maybe_downcast_numeric(result: ExtensionArray, dtype: DtypeObj, do_round: bool = ...) -> ArrayLike:
    ...

def maybe_downcast_numeric(result: ArrayLike, dtype: DtypeObj, do_round: bool = ...) -> ArrayLike:
    """
    Subset of maybe_downcast_to_dtype restricted to numeric dtypes.

    Parameters
    ----------
    result : ndarray or ExtensionArray
    dtype : np.dtype or ExtensionDtype
    do_round : bool

    Returns
    -------
    ndarray or ExtensionArray
    """
    ...

def maybe_cast_pointwise_result(result: ArrayLike, dtype: DtypeObj, numeric_only: bool = ..., same_dtype: bool = ...) -> ArrayLike:
    """
    Try casting result of a pointwise operation back to the original dtype if
    appropriate.

    Parameters
    ----------
    result : array-like
        Result to cast.
    dtype : np.dtype or ExtensionDtype
        Input Series from which result was calculated.
    numeric_only : bool, default False
        Whether to cast only numerics or datetimes as well.
    same_dtype : bool, default True
        Specify dtype when calling _from_sequence

    Returns
    -------
    result : array-like
        result maybe casted to the dtype.
    """
    ...

def maybe_cast_to_extension_array(cls: type[ExtensionArray], obj: ArrayLike, dtype: ExtensionDtype | None = ...) -> ArrayLike:
    """
    Call to `_from_sequence` that returns the object unchanged on Exception.

    Parameters
    ----------
    cls : class, subclass of ExtensionArray
    obj : arraylike
        Values to pass to cls._from_sequence
    dtype : ExtensionDtype, optional

    Returns
    -------
    ExtensionArray or obj
    """
    ...

@overload
def ensure_dtype_can_hold_na(dtype: np.dtype) -> np.dtype:
    ...

@overload
def ensure_dtype_can_hold_na(dtype: ExtensionDtype) -> ExtensionDtype:
    ...

def ensure_dtype_can_hold_na(dtype: DtypeObj) -> DtypeObj:
    """
    If we have a dtype that cannot hold NA values, find the best match that can.
    """
    ...

def maybe_promote(dtype: np.dtype, fill_value=...): # -> tuple[dtype[object_], float] | tuple[dtype[Unknown], object | Unknown | float | Literal[0, False]] | tuple[<subclass of dtype and CategoricalDtype>, float] | tuple[Type[object], NDArray[object_]] | tuple[dtype[Unknown], Any] | tuple[dtype[Unknown], datetime64] | tuple[dtype[object_], <subclass of float and date> | <subclass of float and str> | <subclass of <subclass of <subclass of float and date> and datetime> and str> | float | <subclass of <subclass of float and date> and datetime>] | tuple[dtype[object_] | dtype[float64] | dtype[Unknown], float | object_ | float64 | Unknown]:
    """
    Find the minimal dtype that can hold both the given dtype and fill_value.

    Parameters
    ----------
    dtype : np.dtype
    fill_value : scalar, default np.nan

    Returns
    -------
    dtype
        Upcasted from dtype argument if necessary.
    fill_value
        Upcasted from fill_value argument if necessary.

    Raises
    ------
    ValueError
        If fill_value is a non-scalar and dtype is not object.
    """
    ...

def infer_dtype_from(val, pandas_dtype: bool = ...) -> tuple[DtypeObj, Any]:
    """
    Interpret the dtype from a scalar or array.

    Parameters
    ----------
    val : object
    pandas_dtype : bool, default False
        whether to infer dtype including pandas extension types.
        If False, scalar/array belongs to pandas extension types is inferred as
        object
    """
    ...

def infer_dtype_from_scalar(val, pandas_dtype: bool = ...) -> tuple[DtypeObj, Any]:
    """
    Interpret the dtype from a scalar.

    Parameters
    ----------
    pandas_dtype : bool, default False
        whether to infer dtype including pandas extension types.
        If False, scalar belongs to pandas extension types is inferred as
        object
    """
    ...

def dict_compat(d: dict[Scalar, Scalar]) -> dict[Scalar, Scalar]:
    """
    Convert datetimelike-keyed dicts to a Timestamp-keyed dict.

    Parameters
    ----------
    d: dict-like object

    Returns
    -------
    dict
    """
    ...

def infer_dtype_from_array(arr, pandas_dtype: bool = ...) -> tuple[DtypeObj, ArrayLike]:
    """
    Infer the dtype from an array.

    Parameters
    ----------
    arr : array
    pandas_dtype : bool, default False
        whether to infer dtype including pandas extension types.
        If False, array belongs to pandas extension types
        is inferred as object

    Returns
    -------
    tuple (numpy-compat/pandas-compat dtype, array)

    Notes
    -----
    if pandas_dtype=False. these infer to numpy dtypes
    exactly with the exception that mixed / object dtypes
    are not coerced by stringifying or conversion

    if pandas_dtype=True. datetime64tz-aware/categorical
    types will retain there character.

    Examples
    --------
    >>> np.asarray([1, '1'])
    array(['1', '1'], dtype='<U21')

    >>> infer_dtype_from_array([1, '1'])
    (dtype('O'), [1, '1'])
    """
    ...

def maybe_upcast(values: NumpyArrayT, fill_value: Scalar = ..., copy: bool = ...) -> tuple[NumpyArrayT, Scalar]:
    """
    Provide explicit type promotion and coercion.

    Parameters
    ----------
    values : np.ndarray
        The array that we may want to upcast.
    fill_value : what we want to fill with
    copy : bool, default True
        If True always make a copy even if no upcast is required.

    Returns
    -------
    values: np.ndarray
        the original array, possibly upcast
    fill_value:
        the fill value, possibly upcast
    """
    ...

def invalidate_string_dtypes(dtype_set: set[DtypeObj]) -> None:
    """
    Change string like dtypes to object for
    ``DataFrame.select_dtypes()``.
    """
    ...

def coerce_indexer_dtype(indexer, categories) -> np.ndarray:
    """coerce the indexer input array to the smallest dtype possible"""
    ...

def soft_convert_objects(values: np.ndarray, datetime: bool = ..., numeric: bool = ..., timedelta: bool = ..., period: bool = ..., copy: bool = ...) -> ArrayLike:
    """
    Try to coerce datetime, timedelta, and numeric object-dtype columns
    to inferred dtype.

    Parameters
    ----------
    values : np.ndarray[object]
    datetime : bool, default True
    numeric: bool, default True
    timedelta : bool, default True
    period : bool, default True
    copy : bool, default True

    Returns
    -------
    np.ndarray or ExtensionArray
    """
    ...

def convert_dtypes(input_array: ArrayLike, convert_string: bool = ..., convert_integer: bool = ..., convert_boolean: bool = ..., convert_floating: bool = ...) -> DtypeObj:
    """
    Convert objects to best possible type, and optionally,
    to types supporting ``pd.NA``.

    Parameters
    ----------
    input_array : ExtensionArray or np.ndarray
    convert_string : bool, default True
        Whether object dtypes should be converted to ``StringDtype()``.
    convert_integer : bool, default True
        Whether, if possible, conversion can be done to integer extension types.
    convert_boolean : bool, defaults True
        Whether object dtypes should be converted to ``BooleanDtypes()``.
    convert_floating : bool, defaults True
        Whether, if possible, conversion can be done to floating extension types.
        If `convert_integer` is also True, preference will be give to integer
        dtypes if the floats can be faithfully casted to integers.

    Returns
    -------
    np.dtype, or ExtensionDtype
    """
    ...

def maybe_infer_to_datetimelike(value: np.ndarray) -> np.ndarray | DatetimeArray | TimedeltaArray | PeriodArray | IntervalArray:
    """
    we might have a array (or single object) that is datetime like,
    and no dtype is passed don't change the value unless we find a
    datetime/timedelta set

    this is pretty strict in that a datetime/timedelta is REQUIRED
    in addition to possible nulls/string likes

    Parameters
    ----------
    value : np.ndarray[object]

    Returns
    -------
    np.ndarray, DatetimeArray, TimedeltaArray, PeriodArray, or IntervalArray

    """
    ...

def maybe_cast_to_datetime(value: ExtensionArray | np.ndarray | list, dtype: DtypeObj | None) -> ExtensionArray | np.ndarray:
    """
    try to cast the array/value to a datetimelike dtype, converting float
    nan to iNaT

    We allow a list *only* when dtype is not None.
    """
    ...

def sanitize_to_nanoseconds(values: np.ndarray, copy: bool = ...) -> np.ndarray:
    """
    Safely convert non-nanosecond datetime64 or timedelta64 values to nanosecond.
    """
    ...

def find_result_type(left: ArrayLike, right: Any) -> DtypeObj:
    """
    Find the type/dtype for a the result of an operation between these objects.

    This is similar to find_common_type, but looks at the objects instead
    of just their dtypes. This can be useful in particular when one of the
    objects does not have a `dtype`.

    Parameters
    ----------
    left : np.ndarray or ExtensionArray
    right : Any

    Returns
    -------
    np.dtype or ExtensionDtype

    See also
    --------
    find_common_type
    numpy.result_type
    """
    ...

def common_dtype_categorical_compat(objs: list[Index | ArrayLike], dtype: DtypeObj) -> DtypeObj:
    """
    Update the result of find_common_type to account for NAs in a Categorical.

    Parameters
    ----------
    objs : list[np.ndarray | ExtensionArray | Index]
    dtype : np.dtype or ExtensionDtype

    Returns
    -------
    np.dtype or ExtensionDtype
    """
    ...

@overload
def find_common_type(types: list[np.dtype]) -> np.dtype:
    ...

@overload
def find_common_type(types: list[ExtensionDtype]) -> DtypeObj:
    ...

@overload
def find_common_type(types: list[DtypeObj]) -> DtypeObj:
    ...

def find_common_type(types): # -> DtypeObj | dtype[object_] | dtype[datetime64] | dtype[timedelta64] | dtype[Any]:
    """
    Find a common data type among the given dtypes.

    Parameters
    ----------
    types : list of dtypes

    Returns
    -------
    pandas extension or numpy dtype

    See Also
    --------
    numpy.find_common_type

    """
    ...

def construct_2d_arraylike_from_scalar(value: Scalar, length: int, width: int, dtype: np.dtype, copy: bool) -> np.ndarray:
    ...

def construct_1d_arraylike_from_scalar(value: Scalar, length: int, dtype: DtypeObj | None) -> ArrayLike:
    """
    create a np.ndarray / pandas type of specified shape and dtype
    filled with values

    Parameters
    ----------
    value : scalar value
    length : int
    dtype : pandas_dtype or np.dtype

    Returns
    -------
    np.ndarray / pandas type of length, filled with value

    """
    ...

def construct_1d_object_array_from_listlike(values: Sized) -> np.ndarray:
    """
    Transform any list-like object in a 1-dimensional numpy array of object
    dtype.

    Parameters
    ----------
    values : any iterable which has a len()

    Raises
    ------
    TypeError
        * If `values` does not have a len()

    Returns
    -------
    1-dimensional numpy array of dtype object
    """
    ...

def maybe_cast_to_integer_array(arr: list | np.ndarray, dtype: np.dtype, copy: bool = ...) -> np.ndarray:
    """
    Takes any dtype and returns the casted version, raising for when data is
    incompatible with integer/unsigned integer dtypes.

    Parameters
    ----------
    arr : np.ndarray or list
        The array to cast.
    dtype : np.dtype
        The integer dtype to cast the array to.
    copy: bool, default False
        Whether to make a copy of the array before returning.

    Returns
    -------
    ndarray
        Array of integer or unsigned integer dtype.

    Raises
    ------
    OverflowError : the dtype is incompatible with the data
    ValueError : loss of precision has occurred during casting

    Examples
    --------
    If you try to coerce negative values to unsigned integers, it raises:

    >>> pd.Series([-1], dtype="uint64")
    Traceback (most recent call last):
        ...
    OverflowError: Trying to coerce negative values to unsigned integers

    Also, if you try to coerce float values to integers, it raises:

    >>> maybe_cast_to_integer_array([1, 2, 3.5], dtype=np.dtype("int64"))
    Traceback (most recent call last):
        ...
    ValueError: Trying to coerce float values to integers
    """
    ...

def can_hold_element(arr: ArrayLike, element: Any) -> bool:
    """
    Can we do an inplace setitem with this element in an array with this dtype?

    Parameters
    ----------
    arr : np.ndarray or ExtensionArray
    element : Any

    Returns
    -------
    bool
    """
    ...

def np_can_hold_element(dtype: np.dtype, element: Any) -> Any:
    """
    Raise if we cannot losslessly set this element into an ndarray with this dtype.

    Specifically about places where we disagree with numpy.  i.e. there are
    cases where numpy will raise in doing the setitem that we do not check
    for here, e.g. setting str "X" into a numeric ndarray.

    Returns
    -------
    Any
        The element, potentially cast to the dtype.

    Raises
    ------
    ValueError : If we cannot losslessly store this element with this dtype.
    """
    ...

class LossySetitemError(Exception):
    """
    Raised when trying to do a __setitem__ on an np.ndarray that is not lossless.
    """
    ...





"""
This type stub file was generated by pyright.
"""

import pandas.core.config_init
from __future__ import annotations
from pandas.compat import is_numpy_dev as _is_numpy_dev
from pandas._config import describe_option, get_option, option_context, options, reset_option, set_option
from pandas.core.api import ArrowDtype, BooleanDtype, Categorical, CategoricalDtype, CategoricalIndex, DataFrame, DateOffset, DatetimeIndex, DatetimeTZDtype, Flags, Float32Dtype, Float64Dtype, Grouper, Index, IndexSlice, Int16Dtype, Int32Dtype, Int64Dtype, Int8Dtype, Interval, IntervalDtype, IntervalIndex, MultiIndex, NA, NaT, NamedAgg, Period, PeriodDtype, PeriodIndex, RangeIndex, Series, StringDtype, Timedelta, TimedeltaIndex, Timestamp, UInt16Dtype, UInt32Dtype, UInt64Dtype, UInt8Dtype, array, bdate_range, date_range, factorize, interval_range, isna, isnull, notna, notnull, period_range, set_eng_float_format, timedelta_range, to_datetime, to_numeric, to_timedelta, unique, value_counts
from pandas.core.arrays.sparse import SparseDtype
from pandas.tseries.api import infer_freq
from pandas.tseries import offsets
from pandas.core.computation.api import eval
from pandas.core.reshape.api import concat, crosstab, cut, from_dummies, get_dummies, lreshape, melt, merge, merge_asof, merge_ordered, pivot, pivot_table, qcut, wide_to_long
from pandas import api, arrays, errors, io, plotting, testing, tseries
from pandas.util._print_versions import show_versions
from pandas.io.api import ExcelFile, ExcelWriter, HDFStore, read_clipboard, read_csv, read_excel, read_feather, read_fwf, read_gbq, read_hdf, read_html, read_json, read_orc, read_parquet, read_pickle, read_sas, read_spss, read_sql, read_sql_query, read_sql_table, read_stata, read_table, read_xml, to_pickle
from pandas.io.json import _json_normalize as json_normalize
from pandas.util._tester import test
from pandas._version import get_versions

__docformat__ = ...
_hard_dependencies = ...
_missing_dependencies = ...
if _missing_dependencies:
    ...
v = ...
__version__ = ...
__git_version__ = ...
__deprecated_num_index_names = ...
def __dir__() -> list[str]:
    ...

def __getattr__(name): # -> Type[datetime] | Module("numpy") | Any | Type[SparseArray]:
    ...

__doc__ = ...
__all__ = ["ArrowDtype", "BooleanDtype", "Categorical", "CategoricalDtype", "CategoricalIndex", "DataFrame", "DateOffset", "DatetimeIndex", "DatetimeTZDtype", "ExcelFile", "ExcelWriter", "Flags", "Float32Dtype", "Float64Dtype", "Grouper", "HDFStore", "Index", "IndexSlice", "Int16Dtype", "Int32Dtype", "Int64Dtype", "Int8Dtype", "Interval", "IntervalDtype", "IntervalIndex", "MultiIndex", "NA", "NaT", "NamedAgg", "Period", "PeriodDtype", "PeriodIndex", "RangeIndex", "Series", "SparseDtype", "StringDtype", "Timedelta", "TimedeltaIndex", "Timestamp", "UInt16Dtype", "UInt32Dtype", "UInt64Dtype", "UInt8Dtype", "api", "array", "arrays", "bdate_range", "concat", "crosstab", "cut", "date_range", "describe_option", "errors", "eval", "factorize", "get_dummies", "from_dummies", "get_option", "infer_freq", "interval_range", "io", "isna", "isnull", "json_normalize", "lreshape", "melt", "merge", "merge_asof", "merge_ordered", "notna", "notnull", "offsets", "option_context", "options", "period_range", "pivot", "pivot_table", "plotting", "qcut", "read_clipboard", "read_csv", "read_excel", "read_feather", "read_fwf", "read_gbq", "read_hdf", "read_html", "read_json", "read_orc", "read_parquet", "read_pickle", "read_sas", "read_spss", "read_sql", "read_sql_query", "read_sql_table", "read_stata", "read_table", "read_xml", "reset_option", "set_eng_float_format", "set_option", "show_versions", "test", "testing", "timedelta_range", "to_datetime", "to_numeric", "to_pickle", "to_timedelta", "tseries", "unique", "value_counts", "wide_to_long"]



"""
This type stub file was generated by pyright.
"""

from functools import lru_cache
from typing import Any, Callable, Hashable, Iterable, Mapping, Sequence
from pandas._typing import IndexLabel, StorageOptions
from pandas.util._decorators import doc
from pandas.core.shared_docs import _shared_docs

"""
Utilities for conversion to writer-agnostic Excel representation.
"""
class ExcelCell:
    __fields__ = ...
    __slots__ = ...
    def __init__(self, row: int, col: int, val, style=..., mergestart: int | None = ..., mergeend: int | None = ...) -> None:
        ...
    


class CssExcelCell(ExcelCell):
    def __init__(self, row: int, col: int, val, style: dict | None, css_styles: dict[tuple[int, int], list[tuple[str, Any]]] | None, css_row: int, css_col: int, css_converter: Callable | None, **kwargs) -> None:
        ...
    


class CSSToExcelConverter:
    """
    A callable for converting CSS declarations to ExcelWriter styles

    Supports parts of CSS 2.2, with minimal CSS 3.0 support (e.g. text-shadow),
    focusing on font styling, backgrounds, borders and alignment.

    Operates by first computing CSS styles in a fairly generic
    way (see :meth:`compute_css`) then determining Excel style
    properties from CSS properties (see :meth:`build_xlstyle`).

    Parameters
    ----------
    inherited : str, optional
        CSS declarations understood to be the containing scope for the
        CSS processed by :meth:`__call__`.
    """
    NAMED_COLORS = ...
    VERTICAL_MAP = ...
    BOLD_MAP = ...
    ITALIC_MAP = ...
    FAMILY_MAP = ...
    inherited: dict[str, str] | None
    def __init__(self, inherited: str | None = ...) -> None:
        ...
    
    compute_css = ...
    @lru_cache(maxsize=None)
    def __call__(self, declarations: str | frozenset[tuple[str, str]]) -> dict[str, dict[str, str]]:
        """
        Convert CSS declarations to ExcelWriter style.

        Parameters
        ----------
        declarations : str | frozenset[tuple[str, str]]
            CSS string or set of CSS declaration tuples.
            e.g. "font-weight: bold; background: blue" or
            {("font-weight", "bold"), ("background", "blue")}

        Returns
        -------
        xlstyle : dict
            A style as interpreted by ExcelWriter when found in
            ExcelCell.style.
        """
        ...
    
    def build_xlstyle(self, props: Mapping[str, str]) -> dict[str, dict[str, str]]:
        ...
    
    def build_alignment(self, props: Mapping[str, str]) -> dict[str, bool | str | None]:
        ...
    
    def build_border(self, props: Mapping[str, str]) -> dict[str, dict[str, str | None]]:
        ...
    
    def build_fill(self, props: Mapping[str, str]): # -> dict[str, Unknown] | None:
        ...
    
    def build_number_format(self, props: Mapping[str, str]) -> dict[str, str | None]:
        ...
    
    def build_font(self, props: Mapping[str, str]) -> dict[str, bool | float | str | None]:
        ...
    
    def color_to_excel(self, val: str | None) -> str | None:
        ...
    


class ExcelFormatter:
    """
    Class for formatting a DataFrame to a list of ExcelCells,

    Parameters
    ----------
    df : DataFrame or Styler
    na_rep: na representation
    float_format : str, default None
        Format string for floating point numbers
    cols : sequence, optional
        Columns to write
    header : bool or sequence of str, default True
        Write out column names. If a list of string is given it is
        assumed to be aliases for the column names
    index : bool, default True
        output row names (index)
    index_label : str or sequence, default None
        Column label for index column(s) if desired. If None is given, and
        `header` and `index` are True, then the index names are used. A
        sequence should be given if the DataFrame uses MultiIndex.
    merge_cells : bool, default False
        Format MultiIndex and Hierarchical Rows as merged cells.
    inf_rep : str, default `'inf'`
        representation for np.inf values (which aren't representable in Excel)
        A `'-'` sign will be added in front of -inf.
    style_converter : callable, optional
        This translates Styler styles (CSS) into ExcelWriter styles.
        Defaults to ``CSSToExcelConverter()``.
        It should have signature css_declarations string -> excel style.
        This is only called for body cells.
    """
    max_rows = ...
    max_cols = ...
    def __init__(self, df, na_rep: str = ..., float_format: str | None = ..., cols: Sequence[Hashable] | None = ..., header: Sequence[Hashable] | bool = ..., index: bool = ..., index_label: IndexLabel | None = ..., merge_cells: bool = ..., inf_rep: str = ..., style_converter: Callable | None = ...) -> None:
        ...
    
    @property
    def header_style(self) -> dict[str, dict[str, str | bool]]:
        ...
    
    def get_formatted_cells(self) -> Iterable[ExcelCell]:
        ...
    
    @doc(storage_options=_shared_docs["storage_options"])
    def write(self, writer, sheet_name: str = ..., startrow: int = ..., startcol: int = ..., freeze_panes: tuple[int, int] | None = ..., engine: str | None = ..., storage_options: StorageOptions = ...) -> None:
        """
        writer : path-like, file-like, or ExcelWriter object
            File path or existing ExcelWriter
        sheet_name : str, default 'Sheet1'
            Name of sheet which will contain DataFrame
        startrow :
            upper left cell row to dump data frame
        startcol :
            upper left cell column to dump data frame
        freeze_panes : tuple of integer (length 2), default None
            Specifies the one-based bottommost row and rightmost column that
            is to be frozen
        engine : string, default None
            write engine to use if writer is a path - you can also set this
            via the options ``io.excel.xlsx.writer``, ``io.excel.xls.writer``,
            and ``io.excel.xlsm.writer``.

            .. deprecated:: 1.2.0

                As the `xlwt <https://pypi.org/project/xlwt/>`__ package is no longer
                maintained, the ``xlwt`` engine will be removed in a future
                version of pandas.

        {storage_options}

            .. versionadded:: 1.2.0
        """
        ...
    





"""
This type stub file was generated by pyright.
"""

from typing import Any
from pandas._typing import FilePath, StorageOptions, WriteExcelBuffer
from pandas.io.excel._base import ExcelWriter

class _XlsxStyler:
    STYLE_MAPPING: dict[str, list[tuple[tuple[str, ...], str]]] = ...
    @classmethod
    def convert(cls, style_dict, num_format_str=...): # -> dict[Unknown, Unknown]:
        """
        converts a style_dict to an xlsxwriter format dict

        Parameters
        ----------
        style_dict : style dictionary to convert
        num_format_str : optional number format string
        """
        ...
    


class XlsxWriter(ExcelWriter):
    _engine = ...
    _supported_extensions = ...
    def __init__(self, path: FilePath | WriteExcelBuffer | ExcelWriter, engine: str | None = ..., date_format: str | None = ..., datetime_format: str | None = ..., mode: str = ..., storage_options: StorageOptions = ..., if_sheet_exists: str | None = ..., engine_kwargs: dict[str, Any] | None = ..., **kwargs) -> None:
        ...
    
    @property
    def book(self):
        """
        Book instance of class xlsxwriter.Workbook.

        This attribute can be used to access engine-specific features.
        """
        ...
    
    @property
    def sheets(self) -> dict[str, Any]:
        ...
    





# BoARIO : The Adaptative Regional Input Output model in python.
# Copyright (C) 2022  Samuel Juhel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import coloredlogs
import logging
from functools import lru_cache

__version__ = "v0.4.0b0"

# Create a logger object.
logger = logging.getLogger(__name__)

fieldstyle = {'asctime': {'color': 'green'},
              'levelname': {'bold': True, 'color': 'black'},
              'filename':{'color':'cyan'},
              'funcName':{'color':'blue'}}

levelstyles = {'critical': {'bold': True, 'color': 'red'},
               'debug': {'color': 'green'},
               'error': {'color': 'red'},
               'info': {'color':'magenta'},
               'warning': {'color': 'yellow'}}

coloredlogs.install(level=logging.DEBUG,
                    logger=logger,
                    fmt='%(asctime)s [%(levelname)s] - [%(filename)s > %(funcName)s() > %(lineno)s] - %(message)s',
                    datefmt='%H:%M:%S',
                    field_styles=fieldstyle,
                    level_styles=levelstyles)

DEBUGFORMATTER = logging.Formatter(fmt='%(asctime)s [%(levelname)s] - [%(filename)s > %(funcName)s() > %(lineno)s] - %(message)s',datefmt='%H:%M:%S',)
"""Debug file formatter."""

INFOFORMATTER = logging.Formatter(fmt='%(asctime)s [%(levelname)s] - %(message)s',datefmt='%H:%M:%S',)


@lru_cache(10)
def warn_once(logger, msg: str):
    logger.warning(msg)

"""Log file and stream output formatter."""

#logger.setLevel(logging.DEBUG)
#print("c")
#print(logger.handlers)

# defines the stream handler
#_ch = logging.StreamHandler()  # creates the handler
#_ch.setLevel(logging.DEBUG)  # sets the handler info
#_ch.setFormatter(DEBUGFORMATTER)  # sets the handler formatting

# adds the handler to the global variable: log
#logger.addHandler(_ch)
#logger.propagate = False
#print(logger.handlers)
#print("a")



"""
This type stub file was generated by pyright.
"""

from pandas.io.sas.sas7bdat import SAS7BDATReader

class Parser:
    def __init__(self, parser: SAS7BDATReader) -> None:
        ...
    
    def read(self, nrows: int) -> None:
        ...
    





"""
This type stub file was generated by pyright.
"""

"""
Internal module for console introspection
"""
def get_console_size() -> tuple[int | None, int | None]:
    """
    Return console size as tuple = (width, height).

    Returns (None,None) in non-interactive session.
    """
    ...

def in_interactive_session() -> bool:
    """
    Check if we're running in an interactive shell.

    Returns
    -------
    bool
        True if running under python/ipython interactive shell.
    """
    ...

def in_ipython_frontend() -> bool:
    """
    Check if we're inside an IPython zmq frontend.

    Returns
    -------
    bool
    """
    ...




"""
This type stub file was generated by pyright.
"""

import abc
import numpy as np
from typing import Any, Callable, Dict, Hashable, Iterable, Iterator, TYPE_CHECKING
from pandas._typing import AggFuncType, AggFuncTypeDict, AggObjType, Axis, NDFrameT, npt
from pandas.util._decorators import cache_readonly
from pandas import DataFrame, Index, Series
from pandas.core.groupby import GroupBy
from pandas.core.resample import Resampler
from pandas.core.window.rolling import BaseWindow

if TYPE_CHECKING:
    ...
ResType = Dict[int, Any]
def frame_apply(obj: DataFrame, func: AggFuncType, axis: Axis = ..., raw: bool = ..., result_type: str | None = ..., args=..., kwargs=...) -> FrameApply:
    """construct and return a row or column based frame apply object"""
    ...

class Apply(metaclass=abc.ABCMeta):
    axis: int
    def __init__(self, obj: AggObjType, func, raw: bool, result_type: str | None, args, kwargs) -> None:
        ...
    
    @abc.abstractmethod
    def apply(self) -> DataFrame | Series:
        ...
    
    def agg(self) -> DataFrame | Series | None:
        """
        Provide an implementation for the aggregators.

        Returns
        -------
        Result of aggregation, or None if agg cannot be performed by
        this method.
        """
        ...
    
    def transform(self) -> DataFrame | Series:
        """
        Transform a DataFrame or Series.

        Returns
        -------
        DataFrame or Series
            Result of applying ``func`` along the given axis of the
            Series or DataFrame.

        Raises
        ------
        ValueError
            If the transform function fails or does not transform.
        """
        ...
    
    def transform_dict_like(self, func): # -> DataFrame:
        """
        Compute transform in the case of a dict-like func
        """
        ...
    
    def transform_str_or_callable(self, func) -> DataFrame | Series:
        """
        Compute transform in the case of a string or callable func
        """
        ...
    
    def agg_list_like(self) -> DataFrame | Series:
        """
        Compute aggregation in the case of a list-like argument.

        Returns
        -------
        Result of aggregation.
        """
        ...
    
    def agg_dict_like(self) -> DataFrame | Series:
        """
        Compute aggregation in the case of a dict-like argument.

        Returns
        -------
        Result of aggregation.
        """
        ...
    
    def apply_str(self) -> DataFrame | Series:
        """
        Compute apply in case of a string.

        Returns
        -------
        result: Series or DataFrame
        """
        ...
    
    def apply_multiple(self) -> DataFrame | Series:
        """
        Compute apply in case of a list-like or dict-like.

        Returns
        -------
        result: Series, DataFrame, or None
            Result when self.f is a list-like or dict-like, None otherwise.
        """
        ...
    
    def normalize_dictlike_arg(self, how: str, obj: DataFrame | Series, func: AggFuncTypeDict) -> AggFuncTypeDict:
        """
        Handler for dict-like argument.

        Ensures that necessary columns exist if obj is a DataFrame, and
        that a nested renamer is not passed. Also normalizes to all lists
        when values consists of a mix of list and non-lists.
        """
        ...
    


class NDFrameApply(Apply):
    """
    Methods shared by FrameApply and SeriesApply but
    not GroupByApply or ResamplerWindowApply
    """
    @property
    def index(self) -> Index:
        ...
    
    @property
    def agg_axis(self) -> Index:
        ...
    


class FrameApply(NDFrameApply):
    obj: DataFrame
    @property
    @abc.abstractmethod
    def result_index(self) -> Index:
        ...
    
    @property
    @abc.abstractmethod
    def result_columns(self) -> Index:
        ...
    
    @property
    @abc.abstractmethod
    def series_generator(self) -> Iterator[Series]:
        ...
    
    @abc.abstractmethod
    def wrap_results_for_axis(self, results: ResType, res_index: Index) -> DataFrame | Series:
        ...
    
    @property
    def res_columns(self) -> Index:
        ...
    
    @property
    def columns(self) -> Index:
        ...
    
    @cache_readonly
    def values(self): # -> ndarray[Unknown, Unknown]:
        ...
    
    @cache_readonly
    def dtypes(self) -> Series:
        ...
    
    def apply(self) -> DataFrame | Series:
        """compute the results"""
        ...
    
    def agg(self): # -> DataFrame | Series | Any:
        ...
    
    def apply_empty_result(self): # -> DataFrame | Series:
        """
        we have an empty result; at least 1 axis is 0

        we will try to apply the function to an empty
        series in order to see if this is a reduction function
        """
        ...
    
    def apply_raw(self): # -> DataFrame | Series:
        """apply to the values as a numpy array"""
        ...
    
    def apply_broadcast(self, target: DataFrame) -> DataFrame:
        ...
    
    def apply_standard(self): # -> DataFrame | Series:
        ...
    
    def apply_series_generator(self) -> tuple[ResType, Index]:
        ...
    
    def wrap_results(self, results: ResType, res_index: Index) -> DataFrame | Series:
        ...
    
    def apply_str(self) -> DataFrame | Series:
        ...
    


class FrameRowApply(FrameApply):
    axis = ...
    def apply_broadcast(self, target: DataFrame) -> DataFrame:
        ...
    
    @property
    def series_generator(self): # -> Generator[Series, None, None]:
        ...
    
    @property
    def result_index(self) -> Index:
        ...
    
    @property
    def result_columns(self) -> Index:
        ...
    
    def wrap_results_for_axis(self, results: ResType, res_index: Index) -> DataFrame | Series:
        """return the results for the rows"""
        ...
    


class FrameColumnApply(FrameApply):
    axis = ...
    def apply_broadcast(self, target: DataFrame) -> DataFrame:
        ...
    
    @property
    def series_generator(self): # -> Generator[Series, None, None]:
        ...
    
    @property
    def result_index(self) -> Index:
        ...
    
    @property
    def result_columns(self) -> Index:
        ...
    
    def wrap_results_for_axis(self, results: ResType, res_index: Index) -> DataFrame | Series:
        """return the results for the columns"""
        ...
    
    def infer_to_same_shape(self, results: ResType, res_index: Index) -> DataFrame:
        """infer the results to the same shape as the input object"""
        ...
    


class SeriesApply(NDFrameApply):
    obj: Series
    axis = ...
    def __init__(self, obj: Series, func: AggFuncType, convert_dtype: bool, args, kwargs) -> None:
        ...
    
    def apply(self) -> DataFrame | Series:
        ...
    
    def agg(self): # -> DataFrame | Series:
        ...
    
    def apply_empty_result(self) -> Series:
        ...
    
    def apply_standard(self) -> DataFrame | Series:
        ...
    


class GroupByApply(Apply):
    def __init__(self, obj: GroupBy[NDFrameT], func: AggFuncType, args, kwargs) -> None:
        ...
    
    def apply(self):
        ...
    
    def transform(self):
        ...
    


class ResamplerWindowApply(Apply):
    axis = ...
    obj: Resampler | BaseWindow
    def __init__(self, obj: Resampler | BaseWindow, func: AggFuncType, args, kwargs) -> None:
        ...
    
    def apply(self):
        ...
    
    def transform(self):
        ...
    


def reconstruct_func(func: AggFuncType | None, **kwargs) -> tuple[bool, AggFuncType | None, list[str] | None, npt.NDArray[np.intp] | None]:
    """
    This is the internal function to reconstruct func given if there is relabeling
    or not and also normalize the keyword to get new order of columns.

    If named aggregation is applied, `func` will be None, and kwargs contains the
    column and aggregation function information to be parsed;
    If named aggregation is not applied, `func` is either string (e.g. 'min') or
    Callable, or list of them (e.g. ['min', np.max]), or the dictionary of column name
    and str/Callable/list of them (e.g. {'A': 'min'}, or {'A': [np.min, lambda x: x]})

    If relabeling is True, will return relabeling, reconstructed func, column
    names, and the reconstructed order of columns.
    If relabeling is False, the columns and order will be None.

    Parameters
    ----------
    func: agg function (e.g. 'min' or Callable) or list of agg functions
        (e.g. ['min', np.max]) or dictionary (e.g. {'A': ['min', np.max]}).
    **kwargs: dict, kwargs used in is_multi_agg_with_relabel and
        normalize_keyword_aggregation function for relabelling

    Returns
    -------
    relabelling: bool, if there is relabelling or not
    func: normalized and mangled func
    columns: list of column names
    order: array of columns indices

    Examples
    --------
    >>> reconstruct_func(None, **{"foo": ("col", "min")})
    (True, defaultdict(<class 'list'>, {'col': ['min']}), ('foo',), array([0]))

    >>> reconstruct_func("min")
    (False, 'min', None, None)
    """
    ...

def is_multi_agg_with_relabel(**kwargs) -> bool:
    """
    Check whether kwargs passed to .agg look like multi-agg with relabeling.

    Parameters
    ----------
    **kwargs : dict

    Returns
    -------
    bool

    Examples
    --------
    >>> is_multi_agg_with_relabel(a="max")
    False
    >>> is_multi_agg_with_relabel(a_max=("a", "max"), a_min=("a", "min"))
    True
    >>> is_multi_agg_with_relabel()
    False
    """
    ...

def normalize_keyword_aggregation(kwargs: dict) -> tuple[dict, list[str], npt.NDArray[np.intp]]:
    """
    Normalize user-provided "named aggregation" kwargs.
    Transforms from the new ``Mapping[str, NamedAgg]`` style kwargs
    to the old Dict[str, List[scalar]]].

    Parameters
    ----------
    kwargs : dict

    Returns
    -------
    aggspec : dict
        The transformed kwargs.
    columns : List[str]
        The user-provided keys.
    col_idx_order : List[int]
        List of columns indices.

    Examples
    --------
    >>> normalize_keyword_aggregation({"output": ("input", "sum")})
    (defaultdict(<class 'list'>, {'input': ['sum']}), ('output',), array([0]))
    """
    ...

def relabel_result(result: DataFrame | Series, func: dict[str, list[Callable | str]], columns: Iterable[Hashable], order: Iterable[int]) -> dict[Hashable, Series]:
    """
    Internal function to reorder result if relabelling is True for
    dataframe.agg, and return the reordered result in dict.

    Parameters:
    ----------
    result: Result from aggregation
    func: Dict of (column name, funcs)
    columns: New columns name for relabelling
    order: New order for relabelling

    Examples:
    ---------
    >>> result = DataFrame({"A": [np.nan, 2, np.nan],
    ...       "C": [6, np.nan, np.nan], "B": [np.nan, 4, 2.5]})  # doctest: +SKIP
    >>> funcs = {"A": ["max"], "C": ["max"], "B": ["mean", "min"]}
    >>> columns = ("foo", "aab", "bar", "dat")
    >>> order = [0, 1, 2, 3]
    >>> _relabel_result(result, func, columns, order)  # doctest: +SKIP
    dict(A=Series([2.0, NaN, NaN, NaN], index=["foo", "aab", "bar", "dat"]),
         C=Series([NaN, 6.0, NaN, NaN], index=["foo", "aab", "bar", "dat"]),
         B=Series([NaN, NaN, 2.5, 4.0], index=["foo", "aab", "bar", "dat"]))
    """
    ...

def maybe_mangle_lambdas(agg_spec: Any) -> Any:
    """
    Make new lambdas with unique names.

    Parameters
    ----------
    agg_spec : Any
        An argument to GroupBy.agg.
        Non-dict-like `agg_spec` are pass through as is.
        For dict-like `agg_spec` a new spec is returned
        with name-mangled lambdas.

    Returns
    -------
    mangled : Any
        Same type as the input.

    Examples
    --------
    >>> maybe_mangle_lambdas('sum')
    'sum'
    >>> maybe_mangle_lambdas([lambda: 1, lambda: 2])  # doctest: +SKIP
    [<function __main__.<lambda_0>,
     <function pandas...._make_lambda.<locals>.f(*args, **kwargs)>]
    """
    ...

def validate_func_kwargs(kwargs: dict) -> tuple[list[str], list[str | Callable[..., Any]]]:
    """
    Validates types of user-provided "named aggregation" kwargs.
    `TypeError` is raised if aggfunc is not `str` or callable.

    Parameters
    ----------
    kwargs : dict

    Returns
    -------
    columns : List[str]
        List of user-provied keys.
    func : List[Union[str, callable[...,Any]]]
        List of user-provided aggfuncs

    Examples
    --------
    >>> validate_func_kwargs({'one': 'min', 'two': 'max'})
    (['one', 'two'], ['min', 'max'])
    """
    ...




"""
This type stub file was generated by pyright.
"""

from pandas.io.parsers.readers import TextFileReader, TextParser, read_csv, read_fwf, read_table

__all__ = ["TextFileReader", "TextParser", "read_csv", "read_fwf", "read_table"]



"""
This type stub file was generated by pyright.
"""

def to_numeric(arg, errors=..., downcast=...):
    """
    Convert argument to a numeric type.

    The default return dtype is `float64` or `int64`
    depending on the data supplied. Use the `downcast` parameter
    to obtain other dtypes.

    Please note that precision loss may occur if really large numbers
    are passed in. Due to the internal limitations of `ndarray`, if
    numbers smaller than `-9223372036854775808` (np.iinfo(np.int64).min)
    or larger than `18446744073709551615` (np.iinfo(np.uint64).max) are
    passed in, it is very likely they will be converted to float so that
    they can stored in an `ndarray`. These warnings apply similarly to
    `Series` since it internally leverages `ndarray`.

    Parameters
    ----------
    arg : scalar, list, tuple, 1-d array, or Series
        Argument to be converted.
    errors : {'ignore', 'raise', 'coerce'}, default 'raise'
        - If 'raise', then invalid parsing will raise an exception.
        - If 'coerce', then invalid parsing will be set as NaN.
        - If 'ignore', then invalid parsing will return the input.
    downcast : str, default None
        Can be 'integer', 'signed', 'unsigned', or 'float'.
        If not None, and if the data has been successfully cast to a
        numerical dtype (or if the data was numeric to begin with),
        downcast that resulting data to the smallest numerical dtype
        possible according to the following rules:

        - 'integer' or 'signed': smallest signed int dtype (min.: np.int8)
        - 'unsigned': smallest unsigned int dtype (min.: np.uint8)
        - 'float': smallest float dtype (min.: np.float32)

        As this behaviour is separate from the core conversion to
        numeric values, any errors raised during the downcasting
        will be surfaced regardless of the value of the 'errors' input.

        In addition, downcasting will only occur if the size
        of the resulting data's dtype is strictly larger than
        the dtype it is to be cast to, so if none of the dtypes
        checked satisfy that specification, no downcasting will be
        performed on the data.

    Returns
    -------
    ret
        Numeric if parsing succeeded.
        Return type depends on input.  Series if Series, otherwise ndarray.

    See Also
    --------
    DataFrame.astype : Cast argument to a specified dtype.
    to_datetime : Convert argument to datetime.
    to_timedelta : Convert argument to timedelta.
    numpy.ndarray.astype : Cast a numpy array to a specified type.
    DataFrame.convert_dtypes : Convert dtypes.

    Examples
    --------
    Take separate series and convert to numeric, coercing when told to

    >>> s = pd.Series(['1.0', '2', -3])
    >>> pd.to_numeric(s)
    0    1.0
    1    2.0
    2   -3.0
    dtype: float64
    >>> pd.to_numeric(s, downcast='float')
    0    1.0
    1    2.0
    2   -3.0
    dtype: float32
    >>> pd.to_numeric(s, downcast='signed')
    0    1
    1    2
    2   -3
    dtype: int8
    >>> s = pd.Series(['apple', '1.0', '2', -3])
    >>> pd.to_numeric(s, errors='ignore')
    0    apple
    1      1.0
    2        2
    3       -3
    dtype: object
    >>> pd.to_numeric(s, errors='coerce')
    0    NaN
    1    1.0
    2    2.0
    3   -3.0
    dtype: float64

    Downcasting of nullable integer and floating dtypes is supported:

    >>> s = pd.Series([1, 2, 3], dtype="Int64")
    >>> pd.to_numeric(s, downcast="integer")
    0    1
    1    2
    2    3
    dtype: Int8
    >>> s = pd.Series([1.0, 2.1, 3.0], dtype="Float64")
    >>> pd.to_numeric(s, downcast="float")
    0    1.0
    1    2.1
    2    3.0
    dtype: Float32
    """
    ...




"""
This type stub file was generated by pyright.
"""

from typing import Generator, Iterable, Iterator

"""
Utilities for interpreting CSS from Stylers for formatting non-HTML outputs.
"""
class CSSResolver:
    """
    A callable for parsing and resolving CSS to atomic properties.
    """
    UNIT_RATIOS = ...
    FONT_SIZE_RATIOS = ...
    MARGIN_RATIOS = ...
    BORDER_WIDTH_RATIOS = ...
    BORDER_STYLES = ...
    SIDE_SHORTHANDS = ...
    SIDES = ...
    CSS_EXPANSIONS = ...
    def __call__(self, declarations: str | Iterable[tuple[str, str]], inherited: dict[str, str] | None = ...) -> dict[str, str]:
        """
        The given declarations to atomic properties.

        Parameters
        ----------
        declarations_str : str | Iterable[tuple[str, str]]
            A CSS string or set of CSS declaration tuples
            e.g. "font-weight: bold; background: blue" or
            {("font-weight", "bold"), ("background", "blue")}
        inherited : dict, optional
            Atomic properties indicating the inherited style context in which
            declarations_str is to be resolved. ``inherited`` should already
            be resolved, i.e. valid output of this method.

        Returns
        -------
        dict
            Atomic CSS 2.2 properties.

        Examples
        --------
        >>> resolve = CSSResolver()
        >>> inherited = {'font-family': 'serif', 'font-weight': 'bold'}
        >>> out = resolve('''
        ...               border-color: BLUE RED;
        ...               font-size: 1em;
        ...               font-size: 2em;
        ...               font-weight: normal;
        ...               font-weight: inherit;
        ...               ''', inherited)
        >>> sorted(out.items())  # doctest: +NORMALIZE_WHITESPACE
        [('border-bottom-color', 'blue'),
         ('border-left-color', 'red'),
         ('border-right-color', 'red'),
         ('border-top-color', 'blue'),
         ('font-family', 'serif'),
         ('font-size', '24pt'),
         ('font-weight', 'bold')]
        """
        ...
    
    def size_to_pt(self, in_val, em_pt=..., conversions=...): # -> str:
        ...
    
    def atomize(self, declarations: Iterable) -> Generator[tuple[str, str], None, None]:
        ...
    
    def parse(self, declarations_str: str) -> Iterator[tuple[str, str]]:
        """
        Generates (prop, value) pairs from declarations.

        In a future version may generate parsed tokens from tinycss/tinycss2

        Parameters
        ----------
        declarations_str : str
        """
        ...
    





"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas._typing import npt
from pandas.core.arrays import ExtensionArray

class IndexEngine:
    over_size_threshold: bool
    def __init__(self, values: np.ndarray) -> None:
        ...
    
    def __contains__(self, val: object) -> bool:
        ...
    
    def get_loc(self, val: object) -> int | slice | np.ndarray:
        ...
    
    def sizeof(self, deep: bool = ...) -> int:
        ...
    
    def __sizeof__(self) -> int:
        ...
    
    @property
    def is_unique(self) -> bool:
        ...
    
    @property
    def is_monotonic_increasing(self) -> bool:
        ...
    
    @property
    def is_monotonic_decreasing(self) -> bool:
        ...
    
    @property
    def is_mapping_populated(self) -> bool:
        ...
    
    def clear_mapping(self):
        ...
    
    def get_indexer(self, values: np.ndarray) -> npt.NDArray[np.intp]:
        ...
    
    def get_indexer_non_unique(self, targets: np.ndarray) -> tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]]:
        ...
    


class Float64Engine(IndexEngine):
    ...


class Float32Engine(IndexEngine):
    ...


class Complex128Engine(IndexEngine):
    ...


class Complex64Engine(IndexEngine):
    ...


class Int64Engine(IndexEngine):
    ...


class Int32Engine(IndexEngine):
    ...


class Int16Engine(IndexEngine):
    ...


class Int8Engine(IndexEngine):
    ...


class UInt64Engine(IndexEngine):
    ...


class UInt32Engine(IndexEngine):
    ...


class UInt16Engine(IndexEngine):
    ...


class UInt8Engine(IndexEngine):
    ...


class ObjectEngine(IndexEngine):
    ...


class DatetimeEngine(Int64Engine):
    ...


class TimedeltaEngine(DatetimeEngine):
    ...


class PeriodEngine(Int64Engine):
    ...


class BoolEngine(UInt8Engine):
    ...


class BaseMultiIndexCodesEngine:
    levels: list[np.ndarray]
    offsets: np.ndarray
    def __init__(self, levels: list[np.ndarray], labels: list[np.ndarray], offsets: np.ndarray) -> None:
        ...
    
    def get_indexer(self, target: npt.NDArray[np.object_]) -> npt.NDArray[np.intp]:
        ...
    
    def get_indexer_with_fill(self, target: np.ndarray, values: np.ndarray, method: str, limit: int | None) -> npt.NDArray[np.intp]:
        ...
    


class ExtensionEngine:
    def __init__(self, values: ExtensionArray) -> None:
        ...
    
    def __contains__(self, val: object) -> bool:
        ...
    
    def get_loc(self, val: object) -> int | slice | np.ndarray:
        ...
    
    def get_indexer(self, values: np.ndarray) -> npt.NDArray[np.intp]:
        ...
    
    def get_indexer_non_unique(self, targets: np.ndarray) -> tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]]:
        ...
    
    @property
    def is_unique(self) -> bool:
        ...
    
    @property
    def is_monotonic_increasing(self) -> bool:
        ...
    
    @property
    def is_monotonic_decreasing(self) -> bool:
        ...
    
    def sizeof(self, deep: bool = ...) -> int:
        ...
    
    def clear_mapping(self):
        ...
    





"""
This type stub file was generated by pyright.
"""

from typing import Any, Literal, TYPE_CHECKING
from pandas._typing import FilePath, ReadBuffer, WriteBuffer
from pandas import DataFrame

""" orc compat """
if TYPE_CHECKING:
    ...
def read_orc(path: FilePath | ReadBuffer[bytes], columns: list[str] | None = ..., **kwargs) -> DataFrame:
    """
    Load an ORC object from the file path, returning a DataFrame.

    .. versionadded:: 1.0.0

    Parameters
    ----------
    path : str, path object, or file-like object
        String, path object (implementing ``os.PathLike[str]``), or file-like
        object implementing a binary ``read()`` function. The string could be a URL.
        Valid URL schemes include http, ftp, s3, and file. For file URLs, a host is
        expected. A local file could be:
        ``file://localhost/path/to/table.orc``.
    columns : list, default None
        If not None, only these columns will be read from the file.
    **kwargs
        Any additional kwargs are passed to pyarrow.

    Returns
    -------
    DataFrame

    Notes
    -----
    Before using this function you should read the :ref:`user guide about ORC <io.orc>`
    and :ref:`install optional dependencies <install.warn_orc>`.
    """
    ...

def to_orc(df: DataFrame, path: FilePath | WriteBuffer[bytes] | None = ..., *, engine: Literal["pyarrow"] = ..., index: bool | None = ..., engine_kwargs: dict[str, Any] | None = ...) -> bytes | None:
    """
    Write a DataFrame to the ORC format.

    .. versionadded:: 1.5.0

    Parameters
    ----------
    df : DataFrame
        The dataframe to be written to ORC. Raises NotImplementedError
        if dtype of one or more columns is category, unsigned integers,
        intervals, periods or sparse.
    path : str, file-like object or None, default None
        If a string, it will be used as Root Directory path
        when writing a partitioned dataset. By file-like object,
        we refer to objects with a write() method, such as a file handle
        (e.g. via builtin open function). If path is None,
        a bytes object is returned.
    engine : str, default 'pyarrow'
        ORC library to use. Pyarrow must be >= 7.0.0.
    index : bool, optional
        If ``True``, include the dataframe's index(es) in the file output. If
        ``False``, they will not be written to the file.
        If ``None``, similar to ``infer`` the dataframe's index(es)
        will be saved. However, instead of being saved as values,
        the RangeIndex will be stored as a range in the metadata so it
        doesn't require much space and is faster. Other indexes will
        be included as columns in the file output.
    engine_kwargs : dict[str, Any] or None, default None
        Additional keyword arguments passed to :func:`pyarrow.orc.write_table`.

    Returns
    -------
    bytes if no path argument is provided else None

    Raises
    ------
    NotImplementedError
        Dtype of one or more columns is category, unsigned integers, interval,
        period or sparse.
    ValueError
        engine is not pyarrow.

    Notes
    -----
    * Before using this function you should read the
      :ref:`user guide about ORC <io.orc>` and
      :ref:`install optional dependencies <install.warn_orc>`.
    * This function requires `pyarrow <https://arrow.apache.org/docs/python/>`_
      library.
    * For supported dtypes please refer to `supported ORC features in Arrow
      <https://arrow.apache.org/docs/cpp/orc.html#data-types>`__.
    * Currently timezones in datetime columns are not preserved when a
      dataframe is converted into ORC files.
    """
    ...




"""
This type stub file was generated by pyright.
"""

import contextlib
import pickle as pkl
from typing import Iterator, TYPE_CHECKING
from pandas import DataFrame, Series

"""
Support pre-0.12 series pickle compatibility.
"""
if TYPE_CHECKING:
    ...
def load_reduce(self): # -> None:
    ...

_sparse_msg = ...
class _LoadSparseSeries:
    def __new__(cls) -> Series:
        ...
    


class _LoadSparseFrame:
    def __new__(cls) -> DataFrame:
        ...
    


_class_locations_map = ...
class Unpickler(pkl._Unpickler):
    def find_class(self, module, name): # -> Any:
        ...
    


def load_newobj(self): # -> None:
    ...

def load_newobj_ex(self): # -> None:
    ...

def load(fh, encoding: str | None = ..., is_verbose: bool = ...): # -> Any:
    """
    Load a pickle, with a provided encoding,

    Parameters
    ----------
    fh : a filelike object
    encoding : an optional encoding
    is_verbose : show exception output
    """
    ...

def loads(bytes_object: bytes, *, fix_imports: bool = ..., encoding: str = ..., errors: str = ...): # -> Any:
    """
    Analogous to pickle._loads.
    """
    ...

@contextlib.contextmanager
def patch_pickle() -> Iterator[None]:
    """
    Temporarily patch pickle to use our unpickler.
    """
    ...




"""
This type stub file was generated by pyright.
"""

from typing import Any, Callable

def dumps(obj: Any, ensure_ascii: bool = ..., double_precision: int = ..., indent: int = ..., orient: str = ..., date_unit: str = ..., iso_dates: bool = ..., default_handler: None | Callable[[Any], str | float | bool | list | dict | None] = ...) -> str:
    ...

def loads(s: str, precise_float: bool = ..., numpy: bool = ..., dtype: None = ..., labelled: bool = ...) -> Any:
    ...




"""
This type stub file was generated by pyright.
"""

from contextlib import contextmanager
from typing import Literal

@contextmanager
def assert_produces_warning(expected_warning: type[Warning] | bool | tuple[type[Warning], ...] | None = ..., filter_level: Literal["error", "ignore", "always", "default", "module", "once"] = ..., check_stacklevel: bool = ..., raise_on_extra_warnings: bool = ..., match: str | None = ...): # -> Generator[list[WarningMessage], None, None]:
    """
    Context manager for running code expected to either raise a specific warning,
    multiple specific warnings, or not raise any warnings. Verifies that the code
    raises the expected warning(s), and that it does not raise any other unexpected
    warnings. It is basically a wrapper around ``warnings.catch_warnings``.

    Parameters
    ----------
    expected_warning : {Warning, False, tuple[Warning, ...], None}, default Warning
        The type of Exception raised. ``exception.Warning`` is the base
        class for all warnings. To raise multiple types of exceptions,
        pass them as a tuple. To check that no warning is returned,
        specify ``False`` or ``None``.
    filter_level : str or None, default "always"
        Specifies whether warnings are ignored, displayed, or turned
        into errors.
        Valid values are:

        * "error" - turns matching warnings into exceptions
        * "ignore" - discard the warning
        * "always" - always emit a warning
        * "default" - print the warning the first time it is generated
          from each location
        * "module" - print the warning the first time it is generated
          from each module
        * "once" - print the warning the first time it is generated

    check_stacklevel : bool, default True
        If True, displays the line that called the function containing
        the warning to show were the function is called. Otherwise, the
        line that implements the function is displayed.
    raise_on_extra_warnings : bool, default True
        Whether extra warnings not of the type `expected_warning` should
        cause the test to fail.
    match : str, optional
        Match warning message.

    Examples
    --------
    >>> import warnings
    >>> with assert_produces_warning():
    ...     warnings.warn(UserWarning())
    ...
    >>> with assert_produces_warning(False):
    ...     warnings.warn(RuntimeWarning())
    ...
    Traceback (most recent call last):
        ...
    AssertionError: Caused unexpected warning(s): ['RuntimeWarning'].
    >>> with assert_produces_warning(UserWarning):
    ...     warnings.warn(RuntimeWarning())
    Traceback (most recent call last):
        ...
    AssertionError: Did not see expected warning of class 'UserWarning'.

    ..warn:: This is *not* thread-safe.
    """
    ...

def maybe_produces_warning(warning: type[Warning], condition: bool, **kwargs): # -> _GeneratorContextManager[list[WarningMessage]] | nullcontext[None]:
    """
    Return a context manager that possibly checks a warning based on the condition
    """
    ...

